/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional;

import javax.media.mscontrol.Configuration;
import javax.media.mscontrol.EventType;
import javax.media.mscontrol.Parameter;
import javax.media.mscontrol.Qualifier;
import javax.media.mscontrol.SupportedFeatures;
import javax.media.mscontrol.Value;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.mediagroup.Recorder;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.resource.Action;
import javax.media.mscontrol.resource.Trigger;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.TestBase;

/**
 * Check that each predefined MediaConfig reports accurate values in {@link javax.media.mscontrol.MediaConfig#getSupportedFeatures()}.
 * <br>Most cases check that the items indicating mandatory features are present, and a few check that unwanted items
 * are *not* present, e.g. Recorder.MAX_DURATION in MediaGroupConfig.PLAYER (there is no recorder in
 * this MediaConfig).
 * <br>Items related to optional features are not tested here.
 * <br>The list of testcases, with expected values, is logged by the Logger of this class, at INFO level
 * (set by default).
 */
public class SupportedFeaturesTest extends TestBase {

	static Logger log = Logger.getLogger(SupportedFeaturesTest.class);

	public SupportedFeaturesTest() {
		super();
	}
	public SupportedFeaturesTest(String testMethod) {
		super(testMethod);
	}
	/**
	 * @param configuration
	 * @param item
	 * @param shouldBeSupported Indicate the expected outcome of the test.
	 */
	private  void isSupported(Configuration<?> configuration, Object item, boolean shouldBeSupported) {
		try {
			SupportedFeatures features = myMsControlFactory.getMediaConfig(configuration).getSupportedFeatures();
			boolean isSupported = false;
			if (item instanceof Parameter) {
				isSupported = features.getSupportedParameters().contains(item);
			} else if (item instanceof Value) {
				isSupported = features.getSupportedValues().contains(item);
			} else if (item instanceof Action) {
				isSupported = features.getSupportedActions().contains(item);
			} else if (item instanceof Trigger) {
				isSupported = features.getSupportedTriggers().contains(item);
			} else if (item instanceof EventType) {
				isSupported = features.getSupportedEventTypes().contains(item);
			} else if (item instanceof Qualifier) {
				isSupported = features.getSupportedQualifiers().contains(item);
			} else 
				fail("Unknown feature type: "+item.getClass());
			assertEquals(item+" should be "+(shouldBeSupported ? "supported" : "not supported")+" in "+configuration, shouldBeSupported, isSupported);
			log.info("In "+configuration+", "+item+(shouldBeSupported ? " is" : " is not")+" supported");
		} catch (Exception e) {
			log.error("getting/using SupportedFeatures: "+e.getMessage(), e);
			fail(e.getMessage());
		}		
	}
	/**
	 * Test supported Parameters of {@link MediaGroup#PLAYER}.
	 * <br>Uses {@link javax.media.mscontrol.MediaConfig#getSupportedFeatures()}.
	 * <br>See javadoc of the above for the list of mandatory supported features.
	 */
	@AssertionIds( ids={"javadoc::MediaConfig::2", "javadoc::SupportedFeatures::1"}, desc="getSupportedParameters returns the documented set of Parameters (PLAYER)")
	public void test_2_1_7_1_PLAYERParams() {
		isSupported(MediaGroup.PLAYER, Player.MAX_DURATION, true);
		isSupported(MediaGroup.PLAYER, Player.INTERVAL, true);
		isSupported(MediaGroup.PLAYER, Player.REPEAT_COUNT, true);
		isSupported(MediaGroup.PLAYER, Player.START_OFFSET, true);
	}
	/**
	 * Test supported EventTypes of {@link MediaGroup#PLAYER}.
	 * <br>Uses {@link javax.media.mscontrol.MediaConfig#getSupportedFeatures()}.
	 * <br>See javadoc of the above for the list of mandatory supported features.
	 */
	@AssertionIds( ids={"javadoc::SupportedFeatures::2"}, desc="getSupportedEventTypes returns the documented set of EventTypes (PLAYER)")
	public void test_2_1_7_2_PLAYEREvents() {
		isSupported(MediaGroup.PLAYER, PlayerEvent.PLAY_COMPLETED, true);
		isSupported(MediaGroup.PLAYER, PlayerEvent.END_OF_PLAY_LIST, true);
	}
	/**
	 * Test that some irrelevant features are not included in {@link MediaGroup#PLAYER}.
	 * <br>Uses {@link javax.media.mscontrol.MediaConfig#getSupportedFeatures()}.
	 * <br>Must <b>not</b> be supported: Recorder.MAX_DURATION, SignalDetector.INITIAL_TIMEOUT
	 */
	@AssertionIds( ids={"javadoc::SupportedFeatures::3"}, desc="getSupportedParameters returns no irrelevant Parameter (PLAYER)")
	public void test_2_1_7_3_PLAYERIrrelevant() {
		isSupported(MediaGroup.PLAYER, Recorder.MAX_DURATION, false);
		isSupported(MediaGroup.PLAYER, SignalDetector.INITIAL_TIMEOUT, false);
	}
	/**
	 * Test the features supported by {@link MediaGroup#SIGNALDETECTOR}.
	 * <br>See javadoc of the above for the list of mandatory supported features.
	 * <br>Must NOT be supported: Player.MAX_DURATION, Player.PLAY_COMPLETED, Recorder.MAX_DURATION.
	 */
	@AssertionIds( ids={"javadoc::SupportedFeatures::4"}, desc="getSupportedParameters/EventTypes return the documented values and no irrelevant values (SIGNALDETECTOR)")
	public void test_2_1_7_4_SIGNALDETECTOR() {
		isSupported(MediaGroup.SIGNALDETECTOR, SignalDetector.MAX_DURATION, true);
		isSupported(MediaGroup.SIGNALDETECTOR, SignalDetector.INITIAL_TIMEOUT, true);
		isSupported(MediaGroup.SIGNALDETECTOR, SignalDetector.INTER_SIG_TIMEOUT, true);
		isSupported(MediaGroup.SIGNALDETECTOR, SignalDetector.PATTERN[0], true);
		isSupported(MediaGroup.SIGNALDETECTOR, SignalDetector.PATTERN[15], true);
		isSupported(MediaGroup.SIGNALDETECTOR, SignalDetector.PATTERN[31], true);
		isSupported(MediaGroup.SIGNALDETECTOR, Player.MAX_DURATION, false);
		isSupported(MediaGroup.SIGNALDETECTOR, PlayerEvent.PLAY_COMPLETED, false);
		isSupported(MediaGroup.SIGNALDETECTOR, Recorder.MAX_DURATION, false);
	}	
	/**
	 * Test the features supported by {@link MediaGroup#PLAYER_SIGNALDETECTOR}.
	 * <br>See javadoc of the above for the list of mandatory supported features.
	 * <br>Must <b>not</b> be supported: Recorder.MAX_DURATION, Recorder.AUDIO_CODEC, Recorder.SIGNAL_TRUNCATION_ON,
	 * Recorder.START_BEEP.
	 */
	@AssertionIds( ids={"javadoc::SupportedFeatures::5"}, desc="getSupportedParameters/EventTypes return the documented values and no irrelevant values (PLAYER_SIGNALDETECTOR)")
	public void test_2_1_7_5_PLAYER_SIGNALDETECTOR() {
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, Player.MAX_DURATION, true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, Player.INTERVAL, true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, Player.REPEAT_COUNT, true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, Player.START_OFFSET, true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, PlayerEvent.PLAY_COMPLETED, true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, PlayerEvent.END_OF_PLAY_LIST, true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, SignalDetector.MAX_DURATION, true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, SignalDetector.INITIAL_TIMEOUT, true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, SignalDetector.INTER_SIG_TIMEOUT, true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, SignalDetector.PATTERN[0], true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, SignalDetector.PATTERN[15], true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, SignalDetector.PATTERN[31], true);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, Recorder.MAX_DURATION, false);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, Recorder.AUDIO_CODEC, false);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, Recorder.SIGNAL_TRUNCATION_ON, false);
		isSupported(MediaGroup.PLAYER_SIGNALDETECTOR, Recorder.START_BEEP, false);
	}
	
	/**
	 * Test supported features of {@link MediaGroup#PLAYER_RECORDER_SIGNALDETECTOR}.
	 * <br>See javadoc of the above for the list of mandatory supported features.
	 * <br>Must not be supported: none tested
	 */
	@AssertionIds( ids={"javadoc::SupportedFeatures::6"}, desc="getSupportedParameters/EventTypes return the documented values (PLAYER_RECORDER_SIGNALDETECTOR)")
	public void test_2_1_7_2_PLAYER_RECORDER_SIGNALDETECTOR() {
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, Player.MAX_DURATION, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, Player.INTERVAL, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, Player.REPEAT_COUNT, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, Player.START_OFFSET, true);	
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, PlayerEvent.PLAY_COMPLETED, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, PlayerEvent.END_OF_PLAY_LIST, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, Recorder.MAX_DURATION, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, Recorder.START_BEEP, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, SignalDetector.MAX_DURATION, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, SignalDetector.INITIAL_TIMEOUT, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, SignalDetector.INTER_SIG_TIMEOUT, true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, SignalDetector.PATTERN[0], true);
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, SignalDetector.PATTERN[15], true);		
		isSupported(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR, SignalDetector.PATTERN[31], true);		
	}
}
