/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.join;

import java.io.Serializable;
import java.net.URI;

import javax.media.mscontrol.MediaErr;
import javax.media.mscontrol.MediaEventListener;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.JoinEvent;
import javax.media.mscontrol.join.JoinEventListener;
import javax.media.mscontrol.join.JoinableContainer;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.networkconnection.SdpPortManagerEvent;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.ContinuityChecker;
import com.hp.opencall.jmsc.test.MediaSessionBase;
import com.hp.opencall.jmsc.test.mandatory.functional.mixer.MixerJoinTest;

public class InitJoinTest extends MediaSessionBase {
	
	static Logger log = Logger.getLogger(InitJoinTest.class);
	
	public InitJoinTest() {
		super();
	}
	public InitJoinTest(String method) {
		super(method);
	}
	
	enum TestState {INIT, JOINED, MODIFIED, PLAYED, UNJOINED}
	class Context implements Serializable {
		public Context() {
			myState = TestState.INIT;
		}
		public Context(TestState initState) {
			myState = initState;
		}
		static final long serialVersionUID = 20090310L;
		private TestState myState;
		public TestState getState() {
			return myState;
		}
		void setState(TestState newState) {
			log.debug("Moving from "+myState+" to "+newState);
			myState = newState;
		}
		/** return the state of this context */
		public String toString() {
			return myState.toString();
		}
	}

	/**
	 * Basic announcement play, using the asynchronous
	 * {@link javax.media.mscontrol.join.Joinable#joinInitiate(Direction, javax.media.mscontrol.join.Joinable, Serializable) joinInitiate}
	 * and {@link javax.media.mscontrol.join.Joinable#unjoinInitiate(javax.media.mscontrol.join.Joinable, Serializable) unjoinInitiate}
	 * instead of {@link javax.media.mscontrol.join.Joinable#join(Direction, javax.media.mscontrol.join.Joinable) join}/
	 * {@link javax.media.mscontrol.join.Joinable#unjoin(javax.media.mscontrol.join.Joinable) unjoin}.
	 * <p>The testcase performs the following steps:
	 * <ol>
	 * <li>Create a {@link NetworkConnection} and a {@link MediaGroup}</li>
	 * <li><code>joinInitiate</code> them, and wait for completion signalled on the {@link JoinEventListener}.</li>
	 * <li>setup the NetworkConnection with the modify method, and wait for completion</li>
	 * <li>play an announcement on the Player of the MediaGroup, and wait for completion</li>
	 * <li><code>unjoinInitiate</code> the NetworkConnection and MediaGroup, and wait for completion</li>
	 * </ol>
	 * <p>Use {@link InitJoinTest.Context} to keep track of events.
	 */
	@AssertionIds( ids = { "javadoc::Joinable::9", "javadoc::Joinable::11", "javadoc::JoinEvent::1" , "javadoc::JoinEvent::2", "javadoc::JoinEvent::3" , "javadoc::JoinEvent::4"}, desc = "basic joinInitiate/unjoinInitiate (NC/MG)")
	public void test_2_1_16_1_InitJoinNCMG() throws Exception {
		final NetworkConnection myNC = myMediaSession.createNetworkConnection(NetworkConnection.BASIC);
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		final Context myCtx = new Context();
		myNC.addListener(new JoinEventListener() {
			/* (non-Javadoc)
			 * @see javax.media.mscontrol.StatusEventListener#onEvent(javax.media.mscontrol.StatusEvent)
			 */
			@Override
			public void onEvent(JoinEvent event) {
				log.debug("Received: "+event);
				Context ctx = (Context)event.getContext();
				log.debug("Context: "+ctx);
				TestState state = ctx.getState();
				if (state == TestState.INIT) {
					assertEquals("Bad event id", JoinEvent.JOINED, event.getEventType());
					ctx.setState(TestState.JOINED);
				} else if (state == TestState.PLAYED) {
					assertEquals("Bad event id", JoinEvent.UNJOINED, event.getEventType());
					ctx.setState(TestState.UNJOINED);
				} else 
					fail("Bad TestState: "+state);
				synchronized(myMG) {
					myMG.notify();
				}
			}			
		});
		synchronized(myMG) {
			myNC.joinInitiate(Direction.DUPLEX, myMG, myCtx);
			if (myCtx.getState() != TestState.JOINED)
				myMG.wait(5000);
		}
		myNC.getSdpPortManager().addListener(new MediaEventListener<SdpPortManagerEvent>() {
			@Override
			public void onEvent(SdpPortManagerEvent event) {
				log.debug("Received: "+event);
				assertEquals("processSdpOffer() completed with error: "+event.getError(), MediaErr.NO_ERROR, event.getError());
				assertEquals("Bad TestState", TestState.JOINED, myCtx.getState());
				myCtx.setState(TestState.MODIFIED);
				synchronized(myMG) {
					myMG.notify();
				}
			}
		});
		myNC.getSdpPortManager().processSdpOffer(getProperty("testcase.sipphoneSDP").getBytes());
		synchronized(myMG) {
			myMG.wait(5000);
		}
		assertEquals("Bad TestState after modify", TestState.MODIFIED, myCtx.getState());			

		Player myPlayer = myMG.getPlayer();
		myPlayer.addListener(new MediaEventListener<PlayerEvent>() {
			@Override
			public void onEvent(PlayerEvent event) {
				log.debug("Received: "+event);
				assertEquals("Play completed with error: "+event.getError(), MediaErr.NO_ERROR, event.getError());
				assertEquals("Bad TestState", TestState.MODIFIED, myCtx.getState());
				myCtx.setState(TestState.PLAYED);
				synchronized(myMG) {
					myMG.notify();
				}
			}			
		});
		myPlayer.play(URI.create(getProperty("testcase.shortpromptfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		synchronized(myMG) {
			myMG.wait(8000);
		}
		assertEquals("Bad TestState after play", TestState.PLAYED, myCtx.getState());
		synchronized(myMG) {
			myNC.unjoinInitiate(myMG, myCtx);
			if (myCtx.getState() != TestState.UNJOINED)
				myMG.wait(5000);
		}			
	}
	/**
	 * Everything needed to manage an asynchronous join/unjoin between two JoinableContainers.
	 * <p>Use {@link InitJoinTest.Context} and {@link InitJoinTest.TestState}, with a simplified state machine: only JOINED and UNJOINED.
	 */
	public class JoinManager extends Context implements JoinEventListener {
		static final long serialVersionUID = 20090311L;
		final JoinableContainer one, two;
		public JoinManager(JoinableContainer one, JoinableContainer two) {
			super(TestState.UNJOINED); // set the initial state (not the default in this case)
			this.one = one;
			this.two = two;
			one.addListener(this);
		}
		/**
		 * Listener of {@link JoinEvent}, running a two-states state machine.
		 * <p>Verify the values of {@link JoinEvent#getThisJoinable()} and {@link JoinEvent#getOtherJoinable()}
		 */
		/* (non-Javadoc)
		 * @see javax.media.mscontrol.StatusEventListener#onEvent(javax.media.mscontrol.StatusEvent)
		 */
		@Override
		public void onEvent(JoinEvent event) {
			log.debug("Received: "+event);
			assertEquals("Bad \"ThisJoinable\" reference in JoinEvent", one, event.getThisJoinable());
			assertEquals("Bad \"OtherJoinable\" reference in JoinEvent", two, event.getOtherJoinable());
			Context ctx = (Context)event.getContext();
			log.debug("Context: "+ctx);
			TestState state = ctx.getState();
			if (state == TestState.UNJOINED) {
				assertEquals("Bad event id", JoinEvent.JOINED, event.getEventType());
				ctx.setState(TestState.JOINED);
			} else if (state == TestState.JOINED) {
				assertEquals("Bad event id", JoinEvent.UNJOINED, event.getEventType());
				ctx.setState(TestState.UNJOINED);
			} else 
				fail("Bad TestState: "+state);
			synchronized(this) {
				notify();
			}
		}
		/**
		 * Call <code>joinInitiate</code>, with adequate listener, and performs state checking.
		 */
		public void doJoin() throws Exception {
			synchronized(this) {
				one.joinInitiate(Direction.DUPLEX, two, this);
				if (getState() != TestState.JOINED)
					wait(5000);
			}
			assertEquals("Bad TestState after joinInitiate", TestState.JOINED, getState());
		}
		/**
		 * Call <code>unjoinInitiate</code>, with adequate listener, and performs state checking.
		 */
		public void doUnjoin() throws Exception {
			synchronized(this) {
				one.unjoinInitiate(two, this);
				if (getState() != TestState.UNJOINED)
					wait(5000);
			}			
			assertEquals("Bad TestState after unjoinInitiate", TestState.UNJOINED, getState());
		}
	}
	/**
	 * Join two {@link NetworkConnection}s to a {@link MediaMixer}, using {@link javax.media.mscontrol.join.Joinable#joinInitiate(Direction, javax.media.mscontrol.join.Joinable, Serializable) joinInitiate}/
	 * {@link javax.media.mscontrol.join.Joinable#unjoinInitiate(javax.media.mscontrol.join.Joinable, Serializable) unjoinInitiate}.
	 * <p>The join is checked by sending a DTMF between the two NetworkConnections.
	 * <br>This is very similar to {@link MixerJoinTest#test_2_1_6_1_BasicBridge()}, with <code>join</code> replaced
	 * by <code>initJoin</code>.
	 * <p>Uses one {@link InitJoinTest.JoinManager} for each NetworkConnection.
	 */
	@AssertionIds( ids = { "javadoc::Joinable::10", "javadoc::Joinable::12", "javadoc::JoinEventListener::1" ,"javadoc::JoinEventNotifier::1"}, desc = "basic joinInitiate/unjoinInitiate (NC/Mixer)")
	public void test_2_1_16_2_InitJoinNCMixer() throws Exception {
		ContinuityChecker cc = getContinuityChecker(2, false);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		JoinManager mngSender = new JoinManager(cc.getSender(), myMixer);
		mngSender.doJoin();
		JoinManager mngReceiver = new JoinManager(cc.getReceiver(0), myMixer);
		mngReceiver.doJoin();
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(0, new Boolean[]{false, true}));
		mngSender.doUnjoin();
		mngReceiver.doUnjoin();
	}
}
