/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mediagroup;

import java.net.URI;

import javax.media.mscontrol.MediaErr;
import javax.media.mscontrol.MediaEventListener;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.Qualifier;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.resource.RTC;
import javax.media.mscontrol.resource.ResourceEvent;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;

/**
 * Verify the Player parameters
 */
public class PlayerTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(PlayerTest.class);
	
	public PlayerTest() {
		super();
	}
	public PlayerTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * Allowance in duration measurements
	 */
	int tolerance = 800; // allow 0.8 second of error
	/**
	 * Basics: verify that you can play a file.
	 * <br>Verify that error == NO_ERROR
	 * <br>verify that qualifier == END_OF_PLAY_LIST
	 * <p>Requires testcase.shortpromptfile, 3 seconds duration
	 */
	@AssertionIds( ids = { "javadoc::Joinable::1", "javadoc::MediaGroup::1", "javadoc::MediaGroup::6", "javadoc::Player::1", "javadoc::PlayerEvent::15" }, desc = "basic play verifies many assertions")
	public void test_2_1_1_1_Play() throws Exception {
		int timeout = 10000; // must be more that file duration
		NetworkConnection myNC = setupaNC();
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		myNC.join(Direction.DUPLEX, myMG);
		Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> playerEW = new EventWaiter<PlayerEvent>(myPlayer, log);
		myPlayer.play(URI.create(getProperty("testcase.shortpromptfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		PlayerEvent playEvt = playerEW.waitForOKEvent(timeout);
		assertEquals("Bad completion qualifier: ", PlayerEvent.END_OF_PLAY_LIST, playEvt.getQualifier());
	}
	/**
	 * 
	 * Check that {@link Player#MAX_DURATION} is efficient.
	 * <p>
	 * <ol>
	 * <li>Play a file (that must be at least 5 seconds long) and set MAX_DURATION to 2.6 seconds</li>
	 * <li>Verify that the actual time elapsed before the play completion event is 2.6 seconds,
	 * with an allowance of 0.8 seconds.</li>
	 * <li>Verify that the reported duration in the PlayerEvent is 2.6 seconds +/- allowance.</li>
	 * <li>Verify that the PlayerEvent qualifier is {@link PlayerEvent#DURATION_EXCEEDED}.</li>
	 * </ol>
	 */
	@AssertionIds( ids = { "javadoc::Player::4", "javadoc::PlayerEvent::2", "javadoc::PlayerEvent::17" }, desc = "MAX_DURATION sets the play duration")
	public void test_2_1_1_2_MaxDuration() throws Exception {
		int expectedDuration = 2600; // 2.6 seconds
		URI file = URI.create(getProperty("testcase.promptfile"));
		Parameters options = createParameters();
		options.put(Player.MAX_DURATION, expectedDuration);
		long actualDuration = measurePlayDuration(file, options, log);
		checkDuration("Measured play", expectedDuration, actualDuration, tolerance, log);
		long reportedDuration = lastPlayerEvent.getOffset();
		checkDuration("Reported play", expectedDuration, reportedDuration, tolerance, log);
		assertEquals("Bad PlayerEvent qualifier: ", PlayerEvent.DURATION_EXCEEDED, lastPlayerEvent.getQualifier());
	}
	
	/**
	 * Check that {@link Player#START_OFFSET} is efficient.
	 * <br>1 - Play a file, measure the duration
	 * <br>2 - Play the same file with START_OFFSET = 3000, and verify
	 * that the play duration is 3 seconds shorter.
	 * <br>Verify also that the qualifier of the PlayerEvent is {@link PlayerEvent#END_OF_PLAY_LIST}.
	 */
	@AssertionIds( ids = { "javadoc::Player::5" }, desc = "START_OFFSET changes play duration")
	public void test_2_1_1_3_StartOffset() throws Exception {
		URI file = URI.create(getProperty("testcase.promptfile"));
		int offsetValue = 3000;
		long regularDuration = measurePlayDuration(file, null, log);
		Parameters options = createParameters();
		options.put(Player.START_OFFSET, offsetValue);
		long actualDuration = measurePlayDuration(file, options, log);
		long expectedDuration = regularDuration - offsetValue;
		checkDuration("Measured play", expectedDuration, actualDuration, tolerance, log);
		assertEquals("Bad PlayerEvent qualifier: ", PlayerEvent.END_OF_PLAY_LIST, lastPlayerEvent.getQualifier());
	}
	
	/**
	 * Check that {@link Player#REPEAT_COUNT} is efficient.
	 * <br>Play a one-second file with REPEAT_COUNT = 5, and verify
	 * that the play duration is 5 seconds.
	 */
	@AssertionIds( ids = { "javadoc::Player::20" }, desc = "REPEAT_COUNT causes play repetitions")
	public void test_2_1_1_4_RepeatCount() throws Exception {
		int repeatCount = 5;
		URI oneSecFile = URI.create(getProperty("testcase.onesecondfile"));
		Parameters options = createParameters();
		options.put(Player.REPEAT_COUNT, repeatCount);
		long actualDuration = measurePlayDuration(oneSecFile, options, log);
		long expectedDuration = 1000*repeatCount;
		checkDuration("Measured play", expectedDuration, actualDuration, tolerance, log);
	}

	/**
	 * test {@link Player#INTERVAL}.
	 * <br>Play a one-second file with REPEAT_COUNT = 2, and INTERVAL = 3s.
	 * <br>Verify that the play duration is 5 seconds.
	 */
	@AssertionIds( ids = { "javadoc::Player::21" }, desc = "INTERVAL causes increase of play duration")
	public void test_2_1_1_5_Interval() throws Exception {
		int interval = 3000;
		int repeatCount = 2;
		URI oneSecFile = URI.create(getProperty("testcase.onesecondfile"));
		Parameters options = createParameters();
		options.put(Player.REPEAT_COUNT, repeatCount);
		options.put(Player.INTERVAL, interval);
		long actualDuration = measurePlayDuration(oneSecFile, options, log);
		long expectedDuration = 1000*repeatCount+(interval * (repeatCount-1));
		checkDuration("Measured play", expectedDuration, actualDuration, tolerance, log);
	}
	
	/**
	 * Play a file, and run Player.stop() after 5 seconds.
	 * <br>Verify that the qualifier is {@link ResourceEvent#STOPPED}.
	 */
	@AssertionIds( ids = { "javadoc::Player::61", "javadoc::PlayerEvent::18" }, desc = "Player.stop() cause stop of the Player")
	public void test_2_1_1_6_Stop() throws Exception {
		NetworkConnection myNC = setupaNC();
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		myNC.join(Direction.DUPLEX, myMG);
		Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> pwaiter = new EventWaiter<PlayerEvent>(myPlayer, log);
		myPlayer.play(URI.create(getProperty("testcase.promptfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		pwaiter.failIfEvent(5000);
		myPlayer.stop(true);
		PlayerEvent pEvent = pwaiter.waitForOKEvent(3000);
		assertEquals("Wrong PlayerEvent qualifier: ", pEvent.getQualifier(), PlayerEvent.STOPPED); // Confirm the stop 
	}
	/**
	 * Basics: verify that you can play an array of files.
	 * <p>Play 5 files one after the other in one action, to test play(java.net.URI[] streamIDs, RTC[] rtc, Parameters optargs).
	 * <br>Verify the total duration, and verify that the operation ends with an event qualified by <code>END_OF_PLAY_LIST</code>.
	 * </ol>
	 */
	@AssertionIds( ids = { "javadoc::Player::2", "javadoc::PlayerEvent::16" }, desc = "play an URI array")
	public void test_2_1_1_7_PlayURIArray() throws Exception {
	
		URI file = URI.create(getProperty("testcase.onesecondfile"));
		long expectedduration = 5000;
		
		NetworkConnection myNC = setupaNC();
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		myNC.join(Direction.DUPLEX, myMG);
		Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> playerW = new EventWaiter<PlayerEvent>(myPlayer, log);
		Parameters options = createParameters();
		long playDuration = System.currentTimeMillis(); // initialize
		myPlayer.play(new URI[]{file,file,file,file,file}, null, options);
		lastPlayerEvent = playerW.waitForOKEvent(expectedduration*2);
		assertEquals("The Play is not ended with EOD", PlayerEvent.END_OF_PLAY_LIST, lastPlayerEvent.getQualifier());
		playDuration = System.currentTimeMillis() - playDuration; // measure
		checkDuration("Measured play", expectedduration, playDuration, tolerance*3, log);	
	}

	boolean completed = false;	
	/**
	 * Check the behavior of parameter v_stop when the play action is invoked as player is active.
	 * <br>If the BEHAVIOUR_IF_BUSY parameter is set as STOP_IF_BUSY, as play method is involved as this player is active, the 
	 * Player should terminate the first play and starts the second one.
	 * <ol>   
	 * <li>Verify that the qualifier for the first event is {@link ResourceEvent#STOPPED}</li>
	 * <li>Verify that the second event's qualifier is {@link PlayerEvent#END_OF_PLAY_LIST} </li>
	 * </ol>
	 */
	@AssertionIds( ids = { "javadoc::Player::3" }, desc = "second play stops the running play")
	public void test_2_1_1_8_STOP_IF_BUSY() throws Exception {
		int beforeStop = 4000;
		NetworkConnection myNC = setupaNC();
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		myNC.join(Direction.DUPLEX, myMG);
		Player myPlayer = myMG.getPlayer();
		
		myPlayer.addListener(new MediaEventListener<PlayerEvent>() {
			@Override
			public void onEvent(PlayerEvent anEvent) {
				log.debug("Received: "+anEvent);
				completed = true;
				assertEquals("Play completed with error: "+anEvent.getError(), anEvent.getError(), MediaErr.NO_ERROR);
				Qualifier qualifier = anEvent.getQualifier();
				MediaGroup mg = anEvent.getSource().getContainer();
				int counter = ((Integer)mg.getMediaSession().getAttribute("COUNTER")).intValue();
				log.debug("Media Event fired for Play number "+counter);
				synchronized(mg) {
					if (counter == 1) {
						counter++;
						mg.getMediaSession().setAttribute("COUNTER", new Integer(counter));
						assertTrue("Bad completion qualifier: "+qualifier, qualifier.equals(ResourceEvent.STOPPED));
					} else {
						assertTrue("Bad completion qualifier: "+qualifier, qualifier.equals(PlayerEvent.END_OF_PLAY_LIST));
						myMG.notify();
					}
					
				}
			}			
		});
		completed = false;
		myMG.getMediaSession().setAttribute("COUNTER", new Integer(1));
		long playDuration = System.currentTimeMillis(); // initialize
		myPlayer.play(URI.create(getProperty("testcase.promptfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		Thread.sleep(beforeStop);
		assertTrue("2nd Player is fired after "+beforeStop+"ms", !completed);
		synchronized(myMG) {
			Parameters options2 = createParameters();
			options2.put(Player.BEHAVIOUR_IF_BUSY, Player.STOP_IF_BUSY);
			myPlayer.play(URI.create(getProperty("testcase.onesecondfile")), RTC.NO_RTC, options2); // 2nd play as 1st is still active
			myMG.wait(25000);
			playDuration = System.currentTimeMillis()-playDuration; // measure
		}		
		assertTrue("Play operation timedout after 20s", completed);
		long expectedDuration = beforeStop+1000;  // first play lasts "beforestop", second one lasts for onesecond
		checkDuration("Measured play", expectedDuration, playDuration, tolerance, log);
	}
	
}
