/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008-2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mediagroup;

import java.net.URI;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.mediagroup.signals.SignalDetectorEvent;
import javax.media.mscontrol.resource.RTC;
import javax.media.mscontrol.resource.ResourceEvent;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.LoopbackMediaGroup;
import com.hp.opencall.jmsc.test.MediaSessionBase;

public class PromptAndCollectTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(PromptAndCollectTest.class);
	public PromptAndCollectTest() {
		super();
	}
	public PromptAndCollectTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * Test {@link SignalDetector#PROMPT}.
	 * <p>A {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} provides a
	 * Recorder and a SignalDetector that face each other.
	 * <br>The prompt file (argument of {@link SignalDetector#PROMPT PROMPT}) contains a modulated
	 * DTMF.
	 * <br>When the "player" Signal Detection operation starts, the prompt is played,
	 * and the DTMF triggers the SignalDetector as "detector".
	 * <pre>
	 *     SignalDetector     <----->   SignalDetector
	 *  mySigDetWithPrompt              mySigDet
	 * 
	 *               prompt   ------>
	 *               					detected
	 * </pre>
	 * 
	 */	
	@AssertionIds( ids = {"javadoc::SignalDetector:23" }, desc = "when PROMPT is set, the prompt is played before starting DTMF reception")
	public void test_2_1_10_1_PROMPT() throws Exception {
 
		// 1 - Setup
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(true);		
		SignalDetector mySigDetWithPrompt = myMG.getSignalDetector();		
		SignalDetector mySigDet = myMG.getMG1().getSignalDetector();
		EventWaiter<SignalDetectorEvent> dtmfWaiter = new EventWaiter<SignalDetectorEvent>(mySigDet, log);
		
		// 2 - Start DTMF detection
		mySigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		
		// 3 - Start the detection with prompt. The prompt contains a DTMF and should trigger
		//     the "listening" SignalDetector.
		Parameters promptOpt = createParameters();
		promptOpt.put(SignalDetector.PROMPT, new URI(getProperty("testcase.dtmf1")));
		mySigDetWithPrompt.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, promptOpt);
		
		// 4 - Wait for the DTMF detection event
		SignalDetectorEvent evt = dtmfWaiter.waitForOKEvent(5000);
		assertEquals("Bad DTMF received ", "1", evt.getSignalString());
	}
	
	
	/**
	 * Test the barge-in feature of the Signal detector.
	 * <p> This test case is based on {@link PromptAndRecordTest#test_2_1_9_2_Prompt_BargeIn} - a similar 
	 * functional test for recorder.
	 * 
	 * "barge-in" means that the caller interrupts the prompt with a DTMF key, indicating
	 * that he/she wishes to prematurely end the prompt phase, and start the DTMF collection phase
	 * right away.
	 * <br>This method has two steps: first, test with barge-in disabled, and then, test with barge-in enabled.
	 * <p>A {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} provides a
	 * SignalDetector facing a Player and a SignalDetector:
	 * <pre>
	 *           Player (myPlayer)            <------------>           SignalDetector     
	 *   SignalDetector (myPromptDetector)                           (mySigDetWithPrompt)                
	 *               
	 * </pre>
	 * The detection is started with a prompt (using {@link SignalDetector#PROMPT}).
	 * <br>The prompt file contains:
	 * <br>"cago-cago-cago-cago-cago-cago-cago-cago-DTMF2"
	 * <br>If barge-in is not enabled, the prompt file will always be played to the end,
	 * and the DMTF2 can be caught by a SignalDetector that listens on the other end.
	 * <br>If barge-in is enabled, the prompt is stopped by a DTMF1 sent by the Player,
	 * and the SignalDetector remains idle.
	 * <pre>
	 * 1) barge-in disabled:
	 * time
	 *                                                           receiveSignals() with "prompt"
	 *  0.0s                                          <-- cago ---   <-- prompt
	 *  0.5s                                          <-- cago ---
	 *  1.0s   play("dtmf1")  --->     -- DTMF1 -->   <-- cago ---
	 *  1.5s   										  <-- cago ---
	 *  2.0s                           				  <-- cago ---
	 *  2.5s  					                      <-- cago ---
	 *  3.0s                                          <-- cago ---
	 *  3.5s                                          <-- cago ---
	 *  4.0s receiveSignals 						  <-- DTMF2---
	 *  SignalDetectorEvent for DTMF 2<--        
	 *           |                                                   --> SignalDetectorEvent for DTMF 1
	 *           V
	 *     [[ end of test ]]
	 *
	 * 2) barge-in enabled:
	 * time
	 *                                                             receiveSignals() with "prompt"
	 *  0.0s                                          <-- cago ---   <-- prompt
	 *  0.5s                                          <-- cago ---
	 *  1.0s   play("dtmf1")  --->     -- DTMF1 -->   <-- cago --- [[ prompt stopped ]]                                         
	 *  1.5s   									                --> SignalDetectorEvent for DTMF 1
	 *  2.0s                           
	 *  2.5s   receiveSignals <--
	 *  3.0s
	 *  3.5s                                       (silence)  -->
	 *  4.0s receiveSignals 
	 *  ...	wait for 10 seconds, no event				
	 *             |
	 *             V
	 *       [[ end of test ]]
	 * </pre>
	 * Test assumptions:
	 * <br>With barge-in disabled, the test must end by a SignalDetectorEvent with DTMF2.
	 * <br>With barge-in enabled, the test must end by a time out on receiving Signals.
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector:27", "javadoc::MediaGroup::12" }, desc = "When SIGDET_STOPPLAY is set as RTC, barge-in is enabled")
	public void test_2_1_10_2_Prompt_BargeIn() throws Exception {
		
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(true);

		Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> playWaiter = new EventWaiter<PlayerEvent>(myPlayer, log);

		SignalDetector myPromptDetector = myMG.getMG1().getSignalDetector();
		EventWaiter<SignalDetectorEvent> promptWaiter = new EventWaiter<SignalDetectorEvent>(myPromptDetector, log);
		
		SignalDetector mySigDetWithPrompt = myMG.getSignalDetector();
		EventWaiter<SignalDetectorEvent> sdWaiter = new EventWaiter<SignalDetectorEvent>(mySigDetWithPrompt, log);
		
		log.info("test_2_1_10_2_Prompt_BargeIn: step 1: barge-in disabled");
		
		Parameters promptOpt = createParameters();
		promptOpt.put(SignalDetector.PROMPT, new URI(getProperty("testcase.bargein.prompt")));
		mySigDetWithPrompt.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, promptOpt);  // start prompt and wait for one DTMF input

		sdWaiter.failIfEvent(1000); // The SignalDetector should just wait at this stage

		myPlayer.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER); // play the DTMF 1
		playWaiter.waitForOKEvent(3000);

		myPromptDetector.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);	
		SignalDetectorEvent promptDetectedEvt = promptWaiter.waitForOKEvent(5000);

		// We expect to receive DTMF 2, and the recorder must be still running.
		assertEquals("Bad DTMF received: ", "2", promptDetectedEvt.getSignalString());

		// Cleanup before step 2
		mySigDetWithPrompt.stop();
		sdWaiter.discardEvent(3000); // discard stop event
		mySigDetWithPrompt.flushBuffer();
		sdWaiter.waitForOKEvent(3000); // wait for BUFFER_FLUSHED
		
		log.info("test_2_1_10_2_Prompt_BargeIn: step 2: barge-in enabled");

		mySigDetWithPrompt.receiveSignals(1, SignalDetector.NO_PATTERN, new RTC[]{MediaGroup.SIGDET_STOPPLAY}, promptOpt);	// start prompt and wait for one DTMF input

		sdWaiter.failIfEvent(1000); // detecting must not have started at this stage

		myPlayer.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playWaiter.waitForOKEvent(3000);

		myPromptDetector.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);			
		promptWaiter.failIfEvent(10000); // We must not have received a DTMF at this stage

		log.info("test_2_1_10_2_Prompt_BargeIn terminated, releasing");
	}
	
	private void cancelTest(boolean withPrompt, boolean bargeInEnabled) throws Exception {
		// 1 - Setup
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(true);		
		SignalDetector mySigDet = myMG.getSignalDetector();		
		Player myPlayer = myMG.getPlayer();
		EventWaiter<SignalDetectorEvent> dtmfWaiter = new EventWaiter<SignalDetectorEvent>(mySigDet, log);
		
		Parameters sigDetOpts = mySigDet.getContainer().createParameters();
		sigDetOpts.put(SignalDetector.PATTERN[2], "3");
		if (withPrompt) {
			sigDetOpts.put(SignalDetector.PROMPT, URI.create(getProperty("testcase.promptfile")));
		}
		RTC[] rtcs;
		if (bargeInEnabled) {
			rtcs = new RTC[] {new RTC(SignalDetector.PATTERN_MATCH[2], SignalDetector.CANCEL), MediaGroup.SIGDET_STOPPLAY};
		} else {
			rtcs = new RTC[] {new RTC(SignalDetector.PATTERN_MATCH[2], SignalDetector.CANCEL)};
		}
		mySigDet.receiveSignals(4, SignalDetector.NO_PATTERN, rtcs, sigDetOpts);
		myPlayer.play(URI.create(getProperty("testcase.dtmf1-9")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sigEvt = dtmfWaiter.waitForOKEvent(10000);
		assertEquals("Bad SignalDetectorEvent qualifier", ResourceEvent.RTC_TRIGGERED, sigEvt.getQualifier());
		assertEquals("Bad SignalDetectorEvent RTCTrigger", SignalDetector.PATTERN_MATCH[2], sigEvt.getRTCTrigger());
		
	}

	/**
	 * Test that {@link SignalDetector#CANCEL} is efficient.
	 * <p>A {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} provides a
	 * Player and a SignalDetector that face each other.
	 * <br>The SignalDetector is started with the RTC:
	 * <br>(SignalDetector.PATTERN_MATCH[2], SignalDetector.CANCEL)
	 * <br>and SignalDetector.PATTERN[2] is defined by "3".
	 * <p>The Player plays the DTMFs "1", "2", "3", "4", etc.
	 * <br>The SignalDetector must stop with qualifier {@link ResourceEvent#RTC_TRIGGERED}, and RTCTrigger SignalDetector.PATTERN_MATCH[2].
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector:26", "javadoc::SignalDetector::30" }, desc = "CANCEL causes abort of receiveSignals")
	public void test_2_1_10_3_Cancel() throws Exception {
		cancelTest(false, false);
	}
	/**
	 * Test that {@link SignalDetector#CANCEL} is efficient also during the prompt phase.
	 * <p>Same test as {@link #test_2_1_10_3_Cancel()}, with an additional {@link SignalDetector#PROMPT} argument,
	 * and barge-in enabled by {@link MediaGroup#SIGDET_STOPPLAY}.
	 */
	public void test_2_1_10_4_CancelWithPrompt() throws Exception {
		cancelTest(true, true);
	}

	private void clearDigitsTest(boolean clearDigits) throws Exception {
		// 1 - Setup
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(true);		
		SignalDetector mySigDetWithPrompt = myMG.getSignalDetector();		
		EventWaiter<SignalDetectorEvent> dtmfWaiter = new EventWaiter<SignalDetectorEvent>(mySigDetWithPrompt, log);
		Player myPlayer = myMG.getMG1().getPlayer();
		EventWaiter<PlayerEvent> playWaiter = new EventWaiter<PlayerEvent>(myPlayer, log);

		// 2 - Add DTMF 1 to the buffer
		myPlayer.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playWaiter.waitForOKEvent(4000);

		// 3 - Start a prompt and collect
		Parameters promptOpt = createParameters();
		promptOpt.put(SignalDetector.PROMPT, new URI(getProperty("testcase.onesecondfile")));
		
		RTC[] myRtcs = clearDigits ? new RTC[] {new RTC(Player.PLAY_START, SignalDetector.FLUSH_BUFFER)} : RTC.NO_RTC;
		
		mySigDetWithPrompt.receiveSignals(2, SignalDetector.NO_PATTERN, myRtcs, promptOpt);
		
		// 4 - Wait for the end of the prompt
		dtmfWaiter.failIfEvent(3000);

		// 5 - Add DTMF 1 2 3 4 ... to the buffer
		myPlayer.play(URI.create(getProperty("testcase.dtmf1-9")), RTC.NO_RTC, Parameters.NO_PARAMETER);

		// 6 - Wait for the DTMF detection event
		SignalDetectorEvent evt = dtmfWaiter.waitForOKEvent(5000);
		assertEquals("Bad DTMFs received ", clearDigits ? "12" : "11" , evt.getSignalString());
	}
	/**
	 * Test the "cleardigits" feature.
	 * <p>Cleardigits is set with <code>RTC(Player.PLAY_START, SignalDetector.FLUSH_BUFFER)</code>.
	 * <p>The setup is composed of a {@link SignalDetector} named "mySigDetWithPrompt"
	 * facing a {@link Player} named "myPlayer", in a {@link LoopbackMediaGroup}.
	 * <br>Operations:
	 * <pre>
	 *   mySigDetWithPrompt    <----->   myPlayer
	 *   
	 *                         <------   play DTMF 1              
	 * 
	 * receiveSignals(prompt)   ------> [[ digits are cleared here if cleardigit is true ]]
	 * 
	 *          [end of prompt]
	 *               
	 *                         <------   play DTMF 1 2 3 
	 *     getSignalString()
	 * </pre>
	 * <br>When cleardigits is true, getSignalString() returns "12", otherwise it returns "11".
	 */
	public void test_2_1_10_5_Cleardigits() throws Exception {
		clearDigitsTest(false);
		clearDigitsTest(true);
	}
}
