/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008-2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mediagroup;

import java.net.URI;
import java.util.ArrayList;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.mediagroup.Recorder;
import javax.media.mscontrol.mediagroup.RecorderEvent;
import javax.media.mscontrol.mediagroup.SpeechDetectorConstants;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.mediagroup.signals.SignalDetectorEvent;
import javax.media.mscontrol.resource.Action;
import javax.media.mscontrol.resource.RTC;
import javax.media.mscontrol.resource.ResourceEvent;
import javax.media.mscontrol.resource.Trigger;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.LoopbackMediaGroup;
import com.hp.opencall.jmsc.test.MediaSessionBase;
/**
 * Test the traditional "prompt and record" feature, available on the JSR 309
 * Recorder, using options {@link Recorder#PROMPT}, and RTCs like {@link MediaGroup#SIGDET_STOPPLAY} and {@link Recorder#CANCEL}.
 * <br>Note that the basic recorder features are tested by {@link RecorderTest}.
 *
 */
public class PromptAndRecordTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(PromptAndRecordTest.class);
	public PromptAndRecordTest() {
		super();
	}
	public PromptAndRecordTest(String testMethod) {
		super(testMethod);
	}
	/**
	 * Test {@link Recorder#PROMPT}.
	 * <p>A {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} provides a
	 * Recorder and a SignalDetector that face each other.
	 * <br>The prompt file (argument of {@link Recorder#PROMPT PROMPT}) contains a modulated
	 * DTMF.
	 * <br>When the record operation starts, the prompt is played, and the DTMF triggers the SignalDetector.
	 * <pre>
	 *     Recorder           <----->   SignalDetector
	 *  myRecorderWithPrompt              mySigDet
	 * 
	 *               prompt   ------>
	 *               record   <------
	 * </pre>
	 * 
	 */	
	@AssertionIds( ids = {"javadoc::Recorder::26" }, desc = "when PROMPT is set, the prompt is played before recording")
	public void test_2_1_9_1_PROMPT() throws Exception {

		// 1 - Setup
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(true);		
		Recorder myRecorderWithPrompt = myMG.getRecorder();		
		SignalDetector mySigDet = myMG.getMG1().getSignalDetector();
		EventWaiter<SignalDetectorEvent> dtmfWaiter = new EventWaiter<SignalDetectorEvent>(mySigDet, log);

		// 2 - Start DTMF detection
		mySigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		
		// 3 - Start the recording with prompt. The prompt contains a DTMF and should trigger
		//     the SignalDetector.
		Parameters promptOpt = createParameters();
		promptOpt.put(Recorder.PROMPT, URI.create(getProperty("testcase.dtmf1")));
		myRecorderWithPrompt.record(URI.create(getProperty("testcase.recordedfile")), RTC.NO_RTC, promptOpt);
		
		// 4 - Wait for the DTMF detection event
		SignalDetectorEvent evt = dtmfWaiter.waitForOKEvent(5000);
		assertEquals("Bad DTMF received ", "1", evt.getSignalString());
	}
	/**
	 * Test the barge-in feature of the recorder.
	 * <p>"barge-in" means that the caller interrupts the prompt with a DTMF key, indicating
	 * that he/she wishes to prematurely end the prompt phase, and start the recording phase
	 * right away.
	 * <br>This methods has two steps: first, test with barge-in disabled, and then, test with barge-in enabled.
	 * <p>A {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} provides a
	 * Recorder facing a Player and a SignalDetector:
	 * <pre>
	 *           Player (myPlayer)            <------------>           Recorder     
	 *   SignalDetector (mySigDet)                                (myRecorderWithPrompt)                
	 *               
	 * </pre>
	 * The recorder is started with a prompt (using {@link Recorder#PROMPT}).
	 * <br>The prompt file contains:
	 * <br>"cago-cago-cago-cago-cago-cago-cago-cago-DTMF2"
	 * <br>If barge-in is not enabled, the prompt file will always be played to the end,
	 * and the DMTF2 can be caught by a SignalDetector that listens on the other end.
	 * <br>If barge-in is enabled, the prompt is stopped by a DTMF1 sent by the Player,
	 * and the SignalDetector remains idle.
	 * <pre>
	 * 1) barge-in disabled:
	 * time
	 *                                                             record() with "prompt"
	 *  0.0s                                          <-- cago ---   <-- prompt
	 *  0.5s                                          <-- cago ---
	 *  1.0s   play("dtmf1")  --->     -- DTMF1 -->   <-- cago ---
	 *  1.5s   play("onesecond") --->  --- cago -->   <-- cago ---
	 *  2.0s                           --- cago -->   <-- cago ---
	 *  2.5s   receiveSignals <--                     <-- cago ---
	 *  3.0s                                          <-- cago ---
	 *  3.5s                                          <-- cago ---
	 *  4.0s SignalDetectorEvent <--                  <-- DTMF2---
	 *           |                                                   --> record
	 *           V
	 *     [[ end of test ]]
	 *
	 * 2) barge-in enabled:
	 * time
	 *                                                             record() with "prompt"
	 *  0.0s                                          <-- cago ---   <-- prompt
	 *  0.5s                                          <-- cago ---
	 *  1.0s   play("dtmf1")  --->     -- DTMF1 -->   <-- cago --- [[ prompt stopped ]]                                         
	 *  1.5s   play("onesecond") --->  --- cago -->                  --> record
	 *  2.0s                           --- cago -->
	 *  2.5s   receiveSignals <--
	 *  3.0s
	 *  3.5s                                       (silence)  --> RecorderEvent
	 *                                                                |
	 *                                                                V
	 *                                                         [[ end of test ]]
	 * </pre>
	 * Test assumptions:
	 * <br>With barge-in disabled, the test must end by a SignalDetectorEvent with DTMF2.
	 * <br>With barge-in enabled, the test must end by a RecorderEvent.
	 */
	@AssertionIds( ids = { "javadoc::MediaGroup::12", "javadoc::SignalDetector::27" }, desc = "MediaGroupSIGDET_STOPPLAY allows barge-in")
	public void test_2_1_9_2_Prompt_BargeIn() throws Exception {
		
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(true);

		Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> playWaiter = new EventWaiter<PlayerEvent>(myPlayer, log);

		SignalDetector mySigDet = myMG.getMG1().getSignalDetector();
		EventWaiter<SignalDetectorEvent> sigWaiter = new EventWaiter<SignalDetectorEvent>(mySigDet, log);
		
		Recorder myRecorderWithPrompt = myMG.getRecorder();
		EventWaiter<RecorderEvent> recWaiter = new EventWaiter<RecorderEvent>(myRecorderWithPrompt, log);
		
		log.info("test_2_1_9_2_Prompt_BargeIn: step 1: barge-in disabled");
		
		Parameters promptOpt = createParameters();
		promptOpt.put(Recorder.PROMPT, URI.create(getProperty("testcase.bargein.prompt")));
		promptOpt.put(Recorder.SILENCE_TERMINATION_ON, true);
		promptOpt.put(SpeechDetectorConstants.FINAL_TIMEOUT, 2000);
		myRecorderWithPrompt.record(URI.create(getProperty("testcase.recordedfile")), RTC.NO_RTC, promptOpt);

		recWaiter.failIfEvent(1000); // The recorded should just wait at this stage

		myPlayer.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playWaiter.waitForOKEvent(3000);

		myPlayer.play(URI.create(getProperty("testcase.onesecondfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playWaiter.waitForOKEvent(3000);

		mySigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sigEvt = sigWaiter.waitForOKEvent(5000);

		// We expect to receive DTMF 2, and the recorder must be still running.
		assertEquals("Bad DTMF received: ", "2", sigEvt.getSignalString());
		recWaiter.failIfEvent(0);

		// Cleanup before step 2
		myRecorderWithPrompt.stop();
		recWaiter.waitForOKEvent(3000);
		
		log.info("test_2_1_9_2_Prompt_BargeIn: step 2: barge-in enabled");

		myRecorderWithPrompt.record(URI.create(getProperty("testcase.recordedfile")), new RTC[]{MediaGroup.SIGDET_STOPPLAY}, promptOpt);

		recWaiter.failIfEvent(1000); // Recording must not have started at this stage

		myPlayer.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playWaiter.waitForOKEvent(3000);

		myPlayer.play(URI.create(getProperty("testcase.onesecondfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playWaiter.waitForOKEvent(3000);

		mySigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		
		recWaiter.waitForOKEvent(10000);
		
		sigWaiter.failIfEvent(0); // We must not have received a DTMF at this stage

		log.info("test_2_1_9_2_Prompt_BargeIn terminated, releasing");
	}

	private static final String ANY_DTMF = "Any";
	
	private void recordWithRTC(URI prompt, String pattern, Action rtcAction, boolean willMatch) throws Exception {
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(true);
		Recorder myRecorder = myMG.getRecorder();
		EventWaiter<RecorderEvent> recWaiter = new EventWaiter<RecorderEvent>(myRecorder, log);
		Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> playWaiter = new EventWaiter<PlayerEvent>(myPlayer, log);

		Parameters recordOpt = createParameters();
		Trigger rtcTrigger;
		if (pattern.equals(ANY_DTMF)) {
			rtcTrigger = SignalDetector.DETECTION_OF_ONE_SIGNAL;
		} else {
			rtcTrigger = SignalDetector.PATTERN_MATCH[0];
			recordOpt.put(SignalDetector.PATTERN[0], pattern);
		}
		recordOpt.put(Recorder.SILENCE_TERMINATION_ON, true);
		if (prompt != null)
			recordOpt.put(Recorder.PROMPT, prompt);

		RTC[] rtcs;
		if (prompt != null)
			// If we have a prompt, let's enable barge-in as well
			rtcs = new RTC[] { new RTC(rtcTrigger, rtcAction),  MediaGroup.SIGDET_STOPPLAY };
		else
			rtcs = new RTC[] { new RTC(rtcTrigger, rtcAction) };

		myRecorder.record(URI.create(getProperty("testcase.recordedfile")), rtcs, recordOpt);
		recWaiter.failIfEvent(1000);
		
		myPlayer.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playWaiter.waitForOKEvent(3000);
		
		if (willMatch) {
			RecorderEvent recEvent = recWaiter.waitForOKEvent(3000);
			assertEquals("Bad record completion qualifier: ", ResourceEvent.RTC_TRIGGERED, recEvent.getQualifier());
			assertEquals("Bad record completion RTCTrigger: ", rtcTrigger, recEvent.getRTCTrigger());
		} else {
			RecorderEvent recEvent = recWaiter.waitForOKEvent(8000);
			assertEquals("Bad record completion qualifier: ", RecorderEvent.SILENCE, recEvent.getQualifier());
		}
	}
	/**
	 * Test the "cancel" feature of the recorder.
	 * <br>The cancel feature is available through {@link Recorder#CANCEL}.
	 * <br>This RTC action can be triggered by a DTMF pattern.
	 * <p>A {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} provides a
	 * Recorder facing a Player:
	 * <pre>
	 *           Player            <------------>           Recorder     
	 *         myPlayer                                   (myRecorder)                
	 *               
	 * </pre>
	 * <br>The recorder is started with { PATTERN[0] = "1" }, and RTC(PATTERN_MATCH[0], CANCEL).
	 * <br>The player then plays a file containing the DTMF 1.
	 * <br>We check that the recorder then stops with the qualifier {@link ResourceEvent#RTC_TRIGGERED RTC_TRIGGERED},
	 * and the RTCTrigger {@link SignalDetector#PATTERN_MATCH PATTERN_MATCH[0]}.
	 */
	@AssertionIds( ids = {"javadoc::Recorder::30" }, desc = "Action CANCEL cancels the current recording")
	public void test_2_1_9_3_CANCEL() throws Exception {
		recordWithRTC(null, "1", Recorder.CANCEL, true);
	}

	/**
	 * Test that a DTMF will not cause cancellation if it does not match the cancel pattern.
	 * <br>Same testcase as {@link #test_2_1_9_3_CANCEL()}, with a CANCEL pattern of "*".
	 * <br>The record is not canceled, and ends because of silence detection.
	 */
	public void test_2_1_9_4_NoCancelWrongDTMF() throws Exception {
		recordWithRTC(null, "*", Recorder.CANCEL, false);
	}
	/**
	 * Test the cancel feature, while the recorder is in the prompt phase.
	 * <br>Same testcase as {@link #test_2_1_9_3_CANCEL()}, with an additional
	 * {@link Recorder#PROMPT} parameter.
	 */
	public void test_2_1_9_5_CancelWithPrompt() throws Exception {
		recordWithRTC(URI.create(getProperty("testcase.promptfile")), "1", Recorder.CANCEL, true);
	}
	/**
	 * Test the "return key" feature (a record can be stopped by pressing a DTMF).
	 * <p>The return key can be set by a {@link RTC} associating a {@link SignalDetector#PATTERN}
	 *  to {@link Recorder#STOP}.
	 * <p>This testcase checks that "1" can be set as return key.
	 * <p>A {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} provides a
	 * Recorder facing a Player:
	 * <pre>
	 *           Player            <------------>           Recorder     
	 *         myPlayer                                   (myRecorder)                
	 *               
	 * </pre>
	 * <br>The recorder is started with { PATTERN[0] = "1" }, and RTC(PATTERN_MATCH[0], STOP).
	 * <br>The player then plays a file containing the DTMF 1.
	 * <br>We check that the recorder then stops with the qualifier {@link ResourceEvent#RTC_TRIGGERED RTC_TRIGGERED},
	 * and the RTCTrigger {@link SignalDetector#PATTERN_MATCH PATTERN_MATCH[0]}.
	 */
	public void test_2_1_9_6_SingleReturnKey() throws Exception {
		recordWithRTC(null, "1", Recorder.STOP, true);
	}
	/**
	 * Test with multiple digits enabled as return key.
	 * <br>Same test as {@link #test_2_1_9_6_SingleReturnKey()}, with a returnkey(s) of "12345".
	 */
	public void test_2_1_9_7_MultipleReturnKeys() throws Exception {
		recordWithRTC(null, "12345", Recorder.STOP, true);
	}
	/**
	 * Test that the recorder is not stopped by a DTMF that does not match the returnkey.
	 * <br>Same test as {@link #test_2_1_9_6_SingleReturnKey()}, with a returnkey(s) of "23456789".
	 * <br>The record is not stopped, and ends because of silence detection.
	 */
	public void test_2_1_9_8_NonMatchingReturnKey() throws Exception {
		recordWithRTC(null, "23456789", Recorder.STOP, false);
	}

	/**
	 * Test that when no return key is enabled, the record is not stopped by any DTMF.
	 * <p>A {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} provides a
	 * Recorder and a Player that face each other.
	 * <br>After the Recorder is started, the Player plays a file with all DTMFs in sequence.
	 * <br>The record must not stop.
	 */
	public void test_2_1_9_9_NoReturnKey() throws Exception {
		LoopbackMediaGroup lmg = getLoopbackMediaGroup(true);
		Recorder myRecorder = lmg.getRecorder();
		EventWaiter<RecorderEvent> recordEW = new EventWaiter<RecorderEvent>(myRecorder, log);
		Player myPlayer = lmg.getPlayer();
		EventWaiter<PlayerEvent> playEW = new EventWaiter<PlayerEvent>(myPlayer, log);
		myRecorder.record(URI.create(getProperty("testcase.recordedfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		myPlayer.play(URI.create(getProperty("testcase.dtmf1-9")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playEW.waitForOKEvent(20000);
		recordEW.failIfEvent(3000);
	}
	/**
	 * Test with any digit enabled as return key.
	 * <br>Same test as {@link #test_2_1_9_6_SingleReturnKey()}, with a different RTC to stop the
	 * recording: RTC(DETECTION_OF_ONE_SIGNAL, Recorder.STOP).
	 */
	public void test_2_1_9_10_AnyReturnKey() throws Exception {
		recordWithRTC(null, ANY_DTMF, Recorder.STOP, true);
	}
	public void cleardigitsTest(boolean cleardigits) throws Exception {
		LoopbackMediaGroup lmg = getLoopbackMediaGroup(true);
		Recorder myRecorder = lmg.getRecorder();
		EventWaiter<RecorderEvent> recordEW = new EventWaiter<RecorderEvent>(myRecorder, log);
		Player myPlayer = lmg.getPlayer();
		EventWaiter<PlayerEvent> playEW = new EventWaiter<PlayerEvent>(myPlayer, log);
		
		// Send a DTMF that will wait in the buffer.
		myPlayer.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playEW.waitForOKEvent(4000);

		// Start a recording.
		Parameters promptOpt = createParameters();
		promptOpt.put(Recorder.PROMPT, URI.create(getProperty("testcase.promptfile")));
		ArrayList<RTC> rtcs = new ArrayList<RTC>();
		rtcs.add(new RTC(SignalDetector.DETECTION_OF_ONE_SIGNAL, Recorder.STOP));
		rtcs.add(MediaGroup.SIGDET_STOPPLAY);
		if (cleardigits)
			rtcs.add(new RTC(Player.PLAY_START, SignalDetector.FLUSH_BUFFER));
		myRecorder.record(URI.create(getProperty("testcase.recordedfile")), rtcs.toArray(new RTC[rtcs.size()]), promptOpt);

		// If cleardigits, we are in the prompt phase, else in the record phase.
		// In both cases the record should not complete.
		recordEW.failIfEvent(3000);	
		
		// Send another DTMF.
		// If cleardigits, it should interrupt the prompt and start recording,
		// else it stops the recording.
		myPlayer.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playEW.waitForOKEvent(4000);

		if (cleardigits)
			recordEW.failIfEvent(3000);
		else 
			recordEW.waitForOKEvent(1000);
	}
	/**
	 * Test the "cleardigits" feature.
	 * <p>Cleardigits is set with <code>RTC(Player.PLAY_START, SignalDetector.FLUSH_BUFFER)</code>.
	 * <p>Setup: a {@link Player} facing a {@link Recorder} in a {@link LoopbackMediaGroup}.
	 * <pre>
	 *   myRecorder  <------>  myPlayer
	 *               <------   DTMF 1
	 *      record(prompt, barge-in, cleardigits)
	 *         [[ wait 3 second ]]
	 *               <------   DTMF 1
	 *      [[ record stops immediately if cleardigits is set ]]
	 * </pre>
	 * When cleardigits is set, the record does not stop until the timeout.
	 * <p>The testcase is run two times, without and with cleardigits set.
	 */
	public void test_2_1_9_11_Cleardigits() throws Exception {
		cleardigitsTest(false);
		cleardigitsTest(true);
	}		
}
