/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mediagroup;

import java.net.URI;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.mediagroup.Recorder;
import javax.media.mscontrol.mediagroup.RecorderEvent;
import javax.media.mscontrol.mediagroup.SpeechDetectorConstants;
import javax.media.mscontrol.resource.RTC;
import javax.media.mscontrol.resource.Resource;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.LoopbackMediaGroup;
import com.hp.opencall.jmsc.test.MediaSessionBase;

/**
 * Test the Recorder parameters.
 * <br>Note that the "prompt and record" functionality (PROMPT), and
 * all features requiring a RTC, are tested
 * by {@link PromptAndRecordTest}.
 *
 */
public class RecorderTest extends MediaSessionBase {
	
	static Logger log = Logger.getLogger(RecorderTest.class);

	public RecorderTest() {
		super();
	}
	public RecorderTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * Test the basic record feature.
	 * <p>A {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup}
	 * is used to play a file and record it.
	 * <p>The recorded file is then played back, to check its duration.
	 */
	@AssertionIds( ids = {"javadoc::MediaGroup::2", "javadoc::Recorder::1" }, desc = "basic record verifies many assertions")
	public void test_2_1_2_1_Record() throws Exception {
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(false);
		final Recorder myRecorder = myMG.getRecorder();
		final Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> pwaiter = new EventWaiter<PlayerEvent>(myPlayer, log);
		long playDuration = System.currentTimeMillis(); // initialize
		myRecorder.record(URI.create(getProperty("testcase.recordedfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		myPlayer.play(URI.create(getProperty("testcase.shortpromptfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		pwaiter.waitForOKEvent(20000);
		myRecorder.stop();
		playDuration = System.currentTimeMillis() - playDuration; // measure
		long recordDuration = measurePlayDuration(URI.create(getProperty("testcase.recordedfile")), null, log);
		checkDuration("Measured record", playDuration, recordDuration, 1000, log);
	}
	/**
	 * Test {@link Recorder#MAX_DURATION}.
	 * <p>
	 * <ol>
	 * <li>Create a {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} </li>
	 * <li>Play a long file on the LoopbackMediaGroup, and
	 * record with MAX_DURATION at the same time.</li>
	 * <li>Verify that the record stops after the requested time, with qualifier {@link RecorderEvent#DURATION_EXCEEDED}</li>
	 * </ol>
	 */
	@AssertionIds( ids = {"javadoc::Recorder::13" }, desc = "recording is stopped after MAX_DURATION")
	public void test_2_1_2_2_MaxDuration() throws Exception {
		int expectedDuration = 4500;
		int tolerance = 800;

		LoopbackMediaGroup myMG = getLoopbackMediaGroup(false);
		final Recorder myRecorder = myMG.getRecorder();
		EventWaiter<RecorderEvent> rwaiter = new EventWaiter<RecorderEvent>(myRecorder, log);
		final Player myPlayer = myMG.getPlayer();

		Parameters recordOpts = createParameters();
		recordOpts.put(Recorder.MAX_DURATION, expectedDuration);
		myPlayer.play(URI.create(getProperty("testcase.promptfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		myRecorder.record(URI.create(getProperty("testcase.recordedfile")), RTC.NO_RTC, recordOpts);
		RecorderEvent recEvt = rwaiter.waitForOKEvent(6000);
		assertEquals("Wrong qualifier in RecorderEvent: ", RecorderEvent.DURATION_EXCEEDED, recEvt.getQualifier());
		long actualDuration = measurePlayDuration(URI.create(getProperty("testcase.recordedfile")), null, log);
		myPlayer.stop(true);
		checkDuration("Measured record", expectedDuration, actualDuration, tolerance, log);
	}
	
	/**
	 * Test {@link Recorder#APPEND}.
	 * <p>
	 * <ol>
	 * <li>Create a {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} </li>
	 * <li>Play a file, and record it</li>
	 * <li>Repeat this multiple times, with APPEND set to true</li>
	 * <li>Play the resulting recorded file, and verify play duration</li>
	 * </ol>
	 */
	@AssertionIds( ids = {"javadoc::MediaGroup::2", "javadoc::Recorder::2" }, desc = "APPEND=true causes media to be appended in recorded file")
	public void test_2_1_2_3_Append() throws Exception {
		int iterations = 3;
		int tolerance = 800;
		long playduration = 1000; // using testcase.onesecondfile
		long expectedDuration = playduration*iterations;
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(false);
		final Recorder myRecorder = myMG.getRecorder();
		EventWaiter<RecorderEvent> rwaiter = new EventWaiter<RecorderEvent>(myRecorder, log);
		final Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> pwaiter = new EventWaiter<PlayerEvent>(myPlayer, log);
		
		URI promptfile = URI.create(getProperty("testcase.onesecondfile"));
		URI bucket = URI.create(getProperty("testcase.appendrecordedfile"));
		Parameters appendOpt = createParameters();
		appendOpt.put(Recorder.APPEND, true);
		Parameters recordOpt = null; // for the first iteration
		do {
			myRecorder.record(bucket, RTC.NO_RTC, recordOpt);
			recordOpt = appendOpt; // for all other iterations
			myPlayer.play(promptfile, RTC.NO_RTC, Parameters.NO_PARAMETER);
			pwaiter.waitForOKEvent(2000); // wait for end of the play
			myRecorder.stop();
			rwaiter.waitForOKEvent(5000);	
		} while (--iterations > 0);
		long actualDuration = measurePlayDuration(bucket, null, log);
		checkDuration("Appended record file ", expectedDuration, actualDuration, tolerance, log);
	}
	
    /**
     * Check whether parameter {@link Recorder#START_BEEP} is efficient.
     * <p>
     * A looped Media Group has 2 recorders, in this case one acts as "player"
     * and the other as "recorder". One second after the "recorder" started
     * recording, the "player" recorder starts with a beep.
     * <p>
     * The BEEP recorded can be confirmed manually, by listening to the file.
     */
	@AssertionIds( ids = {"javadoc::Recorder::17" }, desc = "when START_BEEP is true, a beep can be heard by caller")
	public void test_2_1_2_4_StartBeep() throws Exception {
		// prepare the player and recorder
		LoopbackMediaGroup myLMG = getLoopbackMediaGroup(false);
		Recorder myBeeper = myLMG.getRecorder();
		EventWaiter<RecorderEvent> beepWaiter = new EventWaiter<RecorderEvent>(myBeeper, log);
		Recorder myRecorderR = myLMG.getMG1().getRecorder();
		EventWaiter<RecorderEvent> rwaiter = new EventWaiter<RecorderEvent>(myRecorderR, log);
		
		// Start the tester recorder, that should record the beep
		myRecorderR.record(new URI(getProperty("testcase.recordedfile2")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		rwaiter.failIfEvent(1000); // Wait a second so that this recorder gets ready
		
		// Start the under-test recorder, that should issue a beep
		Parameters beepOpt = createParameters();
		beepOpt.put(Recorder.START_BEEP, true);
		myBeeper.record(new URI(getProperty("testcase.recordedfile")), RTC.NO_RTC, beepOpt);

		// wait for 3 second and stop the recording.
		beepWaiter.failIfEvent(3000);
		myBeeper.stop();
		myRecorderR.stop();
		rwaiter.waitForOKEvent(5000);
	
		log.info("Please listen to the recorded file to check the start beep: "+getProperty("testcase.recordedfile2"));
	}
	
	/**
	 * Test {@link Recorder#SILENCE_TERMINATION_ON} parameter. 
	 * <p>
	 * <ol>
	 * <li>Create a {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup LoopbackMediaGroup} </li>
	 * <li>Play a file, and record it. 
	 *     The recorder stops by itself if and only if SILENCE_TERMINATION_ON is true</li>
	 * </ol>
	 * Both values of the parameter (false and true) are tested.
	 */
	@AssertionIds( ids = {"javadoc::Recorder::16", "javadoc::RecorderEvent::7" }, desc = "when SILENCE_TERMINATION_ON is true, recorder stops automatically if silence")
	public void test_2_1_2_5_SilenceTerminationOn() throws Exception {
		// Prepare player and recorder
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(false);
		final Recorder myRecorder = myMG.getRecorder();
		EventWaiter<RecorderEvent> rwaiter = new EventWaiter<RecorderEvent>(myRecorder, log);
		final Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> pwaiter = new EventWaiter<PlayerEvent>(myPlayer, log);
		URI promptfile = URI.create(getProperty("testcase.promptfile"));
		URI file = URI.create(getProperty("testcase.recordedfile"));
		Parameters recordOpt = createParameters();
		
		int playduration = 3000;
		int longSilence = 10000;

		// First run with SILENCE_TERMINATION_ON set to false (the default, actually)
		recordOpt.put(Recorder.SILENCE_TERMINATION_ON, false);
		myRecorder.record(file, RTC.NO_RTC, recordOpt);
		myPlayer.play(promptfile, RTC.NO_RTC, Parameters.NO_PARAMETER);
		pwaiter.failIfEvent(playduration);  // play for 3 seconds
		myPlayer.stop(true);
		pwaiter.waitForOKEvent(3000);
		// Even though it is not receiving media any more, the recorder should not stop
		rwaiter.failIfEvent(longSilence); // wait for some seconds		
		myRecorder.stop();
		rwaiter.waitForOKEvent(3000); // catch record termination

		// Second run with SILENCE_TERMINATION_ON set to true
		recordOpt.put(Recorder.SILENCE_TERMINATION_ON, true);
		myRecorder.record(file, RTC.NO_RTC, recordOpt);
		myPlayer.play(promptfile, RTC.NO_RTC, Parameters.NO_PARAMETER);
		pwaiter.failIfEvent(playduration);  // play for 3 seconds
		myPlayer.stop(true);
		pwaiter.waitForOKEvent(3000);
		// The recorder should stop by itself after detecting silence
		RecorderEvent lastRecorderEvent = rwaiter.waitForOKEvent(longSilence); // wait for final timeout
		assertEquals("Recorder was not stopped by final silence:", RecorderEvent.SILENCE, lastRecorderEvent.getQualifier());
	}

	/**
	 * Test {@link SpeechDetectorConstants#FINAL_TIMEOUT} applied to the Recorder.
	 * <p>This test requires that {@link Recorder#SILENCE_TERMINATION_ON} is set to true.
	 * <br>3 values of FINAL_TIMEOUT are tested: 4 seconds, 8 seconds, and {@link Resource#FOR_EVER}.
	 * <p>Test setup: a {@link LoopbackMediaGroup} places a Player and a Recorder in face of each other.
	 * <br>In each step:
	 * <ol>
	 * <li>the recorder is started with a value of FINAL_TIMEOUT</li>
	 * <li>the player plays a message</li>
	 * <li>after 3 seconds, the player is stopped</li>
	 * <li>the record completion is checked, with {@link RecorderEvent#SILENCE}, or no completion for the
	 *  {@link Resource#FOR_EVER} case.
	 * 
	 */
	@AssertionIds( ids = {"javadoc::SpeechDetectorConstants::1" }, desc = "FINAL_TIMEOUT controls the length of a period of silence after callers have spoken to conclude they finished")
	public void test_2_1_2_6_FinalTimeout() throws Exception {
		// Prepare player and recorder
		LoopbackMediaGroup myMG = getLoopbackMediaGroup(false);
		final Recorder myRecorder = myMG.getRecorder();
		EventWaiter<RecorderEvent> rwaiter = new EventWaiter<RecorderEvent>(myRecorder, log);
		final Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> pwaiter = new EventWaiter<PlayerEvent>(myPlayer, log);
		URI promptfile = URI.create(getProperty("testcase.promptfile"));
		URI file = URI.create(getProperty("testcase.recordedfile"));
		Parameters recordOpt = createParameters();
		recordOpt.put(Recorder.SILENCE_TERMINATION_ON, true);
		
		int playduration = 3000;
		int longSilence = 12000;

		// First run with final timeout of 4 seconds
		recordOpt.put(SpeechDetectorConstants.FINAL_TIMEOUT, 4000);
		myRecorder.record(file, RTC.NO_RTC, recordOpt);
		myPlayer.play(promptfile, RTC.NO_RTC, Parameters.NO_PARAMETER);
		pwaiter.failIfEvent(playduration);  // play for 3 seconds
		myPlayer.stop(true);
		pwaiter.waitForOKEvent(3000); 
		RecorderEvent recEvent = rwaiter.waitForOKEvent(longSilence); // wait for final timeout
		assertEquals("Recorder was not stopped by final silence", recEvent.getQualifier(), RecorderEvent.SILENCE);

		// Second run with final timeout of 8 seconds
		recordOpt.put(SpeechDetectorConstants.FINAL_TIMEOUT, 8000);
		myRecorder.record(file, RTC.NO_RTC, recordOpt);
		myPlayer.play(promptfile, RTC.NO_RTC, Parameters.NO_PARAMETER);
		pwaiter.failIfEvent(playduration);  // play for 3 seconds
		myPlayer.stop(true);
		pwaiter.waitForOKEvent(3000); 
		recEvent = rwaiter.waitForOKEvent(longSilence); // wait for final timeout
		assertEquals("Recorder was not stopped by final silence", recEvent.getQualifier(), RecorderEvent.SILENCE);

		// Third run with final timeout of FOREVER
		recordOpt.put(SpeechDetectorConstants.FINAL_TIMEOUT, Resource.FOR_EVER);
		myRecorder.record(file, RTC.NO_RTC, recordOpt);
		myPlayer.play(promptfile, RTC.NO_RTC, Parameters.NO_PARAMETER);
		pwaiter.failIfEvent(playduration);  // play for 3 seconds
		myPlayer.stop(true);
		pwaiter.waitForOKEvent(3000);
		// Even though it is not receiving media any more, the recorder should not stop
		rwaiter.failIfEvent(longSilence); // wait for some seconds		
		myRecorder.stop();
	}
}
