/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mediagroup.signals;

import java.net.URI;

import javax.media.mscontrol.Parameters;

import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.mediagroup.signals.SignalDetectorEvent;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.LoopbackMediaGroup;
import com.hp.opencall.jmsc.test.MediaSessionBase;
import com.hp.opencall.jmsc.test.SipPhoneEmulator;

/**
 * Test if DTMFs are buffered when they should, and flushed when they should.
 */
public class DTMFBufferingTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(DTMFBufferingTest.class);
	
	public DTMFBufferingTest() {
		super();
	}
	public DTMFBufferingTest(String testMethod) {
		super(testMethod);
	}
	/**
	 * Utility class to setup the Player and SignalDetector of a LoopbackMediaGroup.
	 */
	class DTMFCatcher {
		public final Player player;
		public final EventWaiter<PlayerEvent> playerEW;
		public final SignalDetector sigDet;
		public final EventWaiter<SignalDetectorEvent> sdEW;
		public DTMFCatcher() throws Exception  {
			LoopbackMediaGroup lmg = getLoopbackMediaGroup(true);
			player = lmg.getPlayer();
			playerEW = new EventWaiter<PlayerEvent>(player, log);
			sigDet = lmg.getSignalDetector();
			sdEW = new EventWaiter<SignalDetectorEvent>(sigDet, log);			
		}
	}
	/**
	 * Test that DTMFs are buffered by the SignalDetector.
	 * <p>A {@link LoopbackMediaGroup} has a Player facing a SignalDetector.
	 * <ol>
	 * <li>The Player sends a DTMF "1".</li>
	 * <li>The Player sends a DTMF "1" again.</li>
	 * <li>{@link SignalDetector#receiveSignals(int, javax.media.mscontrol.Parameter[], javax.media.mscontrol.resource.RTC[], javax.media.mscontrol.Parameters) receiveSignals}
	 *  is called, and must complete with "11".</li>
	 * </ol>
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector:11", "javadoc::SignalDetector::31", "javadoc::SignalDetectorEvent::1", "javadoc::SignalDetectorEvent::5", "javadoc::SignalDetectorEvent::13" }, desc = "Signals are kept in the signal buffer until they are removed by the application")
	public void test_2_1_12_1_Buffer() throws Exception {
		DTMFCatcher catcher = new DTMFCatcher();
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(5000);
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(5000);
		catcher.sigDet.receiveSignals(2, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sdEvt = catcher.sdEW.waitForOKEvent(5000);
		assertEquals("Bad EventType", SignalDetectorEvent.RECEIVE_SIGNALS_COMPLETED, sdEvt.getEventType());
		assertEquals("Bad Qualifier", SignalDetectorEvent.NUM_SIGNALS_DETECTED, sdEvt.getQualifier());
		assertEquals("Bad DTMF received", "11", sdEvt.getSignalString());		
	}
	/**
	 * Test that DTMFs are buffered by the SignalDetector, between calls to
	 * {@link SignalDetector#receiveSignals(int, javax.media.mscontrol.Parameter[], javax.media.mscontrol.resource.RTC[], javax.media.mscontrol.Parameters) receiveSignals}.
	 * <p>A {@link LoopbackMediaGroup} has a Player facing a SignalDetector.
	 * <ol>
	 * <li>The Player sends a DTMF "1".</li>
	 * <li>{@link SignalDetector#receiveSignals(int, javax.media.mscontrol.Parameter[], javax.media.mscontrol.resource.RTC[], javax.media.mscontrol.Parameters) receiveSignals}
	 * is called, and must complete with "1".</li>
	 * <li>The Player sends a DTMF "1".</li>
	 * <li>The Player sends a DTMF "1" again.</li>
	 * <li><code>receiveSignals</code> is called, and must complete with "11".</li>
	 * </ol>
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector:12" }, desc = "Signals are kept in the signal buffer between calls to receiveSignals")
	public void test_2_1_12_2_BufferBetweenReceive() throws Exception {
		DTMFCatcher catcher = new DTMFCatcher();
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(5000);
		catcher.sigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sdEvt = catcher.sdEW.waitForOKEvent(10000);
		assertEquals("Bad DTMF received", "1", sdEvt.getSignalString());		
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(5000);
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(5000);
		catcher.sigDet.receiveSignals(2, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		sdEvt = catcher.sdEW.waitForOKEvent(5000);
		assertEquals("Bad DTMF received", "11", sdEvt.getSignalString());		
	}
	/**
	 * Test that DTMFs are buffered as soon as the MediaGroup is joined to the NetworkConnection.
	 * <p>A {@link LoopbackMediaGroup} has a Player facing a SignalDetector.
	 * <ol>
	 * <li>The Player sends a DTMF "1".</li>
	 * <li>{@link SignalDetector#receiveSignals(int, javax.media.mscontrol.Parameter[], javax.media.mscontrol.resource.RTC[], javax.media.mscontrol.Parameters) receiveSignals}
	 * is called, <b>after</b> the DTMF was received</li>
	 * </ol>
	 * <p><code>receiveSignals</code> must complete with DTMF "1".
	 */
	public void test_2_1_12_3_BufferBeforeReceive() throws Exception {
		DTMFCatcher catcher = new DTMFCatcher();
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(5000);
		catcher.sigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sdEvt = catcher.sdEW.waitForOKEvent(10000);
		assertEquals("Bad DTMF received", "1", sdEvt.getSignalString());		
	}
	/**
	 * Test that buffered DTMFs can be retrieved one by one.
	 * <p>A {@link LoopbackMediaGroup} has a Player facing a SignalDetector.
	 * <ol>
	 * <li>The Player sends a DTMF "1".</li>
	 * <li>The Player sends a DTMF "1" again. The DTMF buffer now contains "11".</li>
	 * <li>{@link SignalDetector#receiveSignals(int, javax.media.mscontrol.Parameter[], javax.media.mscontrol.resource.RTC[], javax.media.mscontrol.Parameters) receiveSignals}
	 * is called, <b>asking for 1 DTMF</b>, and must complete with "1".</li>
	 * <li><code>receiveSignals</code> is called again, and must complete with "1" again.</li>
	 * </ol>
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector:13" }, desc = "numsignals can limit the number of received signals")
	public void test_2_1_12_4_ReceiveBufferedOneAtATime() throws Exception {
		DTMFCatcher catcher = new DTMFCatcher();
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(5000);
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(5000);
		catcher.sigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sdEvt = catcher.sdEW.waitForOKEvent(5000);
		assertEquals("Bad DTMF received", "1", sdEvt.getSignalString());				
		catcher.sigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		sdEvt = catcher.sdEW.waitForOKEvent(5000);
		assertEquals("Bad DTMF received", "1", sdEvt.getSignalString());				
	}
	/**
	 * Test that {@link SignalDetector#flushBuffer()} is efficient.
	 * <p>A {@link LoopbackMediaGroup} has a Player facing a SignalDetector.
	 * <ol>
	 * <li>The Player sends a DTMF.</li>
	 * <li>The SignalDetector buffer is flushed.</li>
	 * <li>{@link SignalDetector#receiveSignals(int, javax.media.mscontrol.Parameter[], javax.media.mscontrol.resource.RTC[], javax.media.mscontrol.Parameters) receiveSignals}
	 * is called.</li>
	 * </ol>
	 * <p>receiveSignals must <b>not</b> complete.
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector::10" }, desc = "SD can remove all signals from the signal buffer")
	public void test_2_1_12_5_flushBuffer() throws Exception {
		playAndFlushAndPlay(false);
	}

	private void playAndFlushAndPlay(boolean playAgain) throws Exception {
		DTMFCatcher catcher = new DTMFCatcher();
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(5000);
		
		catcher.sigDet.flushBuffer();

		// Wait for the flushBuffer completion event
		catcher.sdEW.waitForOKEvent(4000);
		
		catcher.sigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER); // INITIAL_TIMEOUT is FOREVER by default, so no event to expect
		catcher.sdEW.failIfEvent(3000);
		if (!playAgain)
			return;
		
		catcher.player.play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		catcher.playerEW.waitForOKEvent(20000);
		catcher.sigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sdEvt = catcher.sdEW.waitForOKEvent(10000);
		assertEquals("Bad DTMF received", "1", sdEvt.getSignalString());		
	}
	/**
	 * Test that after a {@link SignalDetector#flushBuffer()}, new DTMFs are buffered.
	 * <p>This testcase runs the same sequence as {@link #test_2_1_12_5_flushBuffer()},
	 * then plays a new DTMF, and then calls <code>receiveSignals</code> again.
	 * <p>The second call to <code>receiveSignals</code> must complete with DTMF "1".
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector::14" }, desc = "buffering continues after flush")
	public void test_2_1_12_6_BufferAfterFlush() throws Exception {
		playAndFlushAndPlay(true);
	}

	/**
	 * Test that DTMFs emitted by a conference participant are buffered.
	 * <p>Uses a {@link SipPhoneEmulator} (named "Marc"), a MediaGroup, and a Mixer.
	 * <br>Setup:
	 * <pre>
	 *                ____________
	 *    _____      | MediaGroup |        
	 *   |     |---->|____________|
	 *   | Marc|                       _______
	 *   |_____|<-------------------->| Mixer |
	 *                                |_______|
	 * </pre>
	 * <br>1 - Marc sends the DTMF "1".
	 * <br>2 - Marc sends the DTMF "1", again.
	 * <br>3 - <code>receiveSignals</code> is called.
	 * <p><code>receiveSignals</code> must complete and return "11".
	 */
	@AssertionIds( ids = { "javadoc::MediaGroup::7" }, desc = "")
	public void test_2_1_12_7_BufferParticipantDTMF() throws Exception {
		// Create and join containers
		SipPhoneEmulator marc = getInbandSipPhoneEmulator("Marc");
		MediaGroup dtmfListenMG = myMediaSession.createMediaGroup(MediaGroup.SIGNALDETECTOR);
		MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		marc.myJoinPoint.join(Direction.DUPLEX, dtmfListenMG);
		marc.myJoinPoint.join(Direction.DUPLEX, myMixer);

		// Prepare the SignalDetector
		SignalDetector dtmfListener = dtmfListenMG.getSignalDetector();
		EventWaiter<SignalDetectorEvent> listenEW = new EventWaiter<SignalDetectorEvent>(dtmfListener, log);

		// 1 - Marc sends the DTMF "1".
		marc.playPrompt(getProperty("testcase.dtmf1"));
		// 2 - Marc sends the DTMF "1" again.
		marc.playPrompt(getProperty("testcase.dtmf1"));
		// 3 - receiveSignals is called
		dtmfListener.receiveSignals(2, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sdEvt = listenEW.waitForOKEvent(10000);
		assertEquals("Bad DTMF received", "11", sdEvt.getSignalString());		
	}

	/**
	 * Test that DTMFs emitted by conference participants, <i>before they enter the conference</i>, are buffered.
	 * <p>Procedure:
	 * <br>1 - Join a {@link SipPhoneEmulator} (named "Marc") to a MediaGroup:
	 * <pre>
	 *                ____________
	 *    _____      | MediaGroup |        
	 *   |     |<--->|____________|
	 *   | Marc|
	 *   |_____| 
	 * </pre>
	 * <br>2 - Marc sends a DTMF "1" that should be caught and buffered by the SignalDetector of
	 * the MediaGroup. At this point, the application does not attempt to receive this DTMF.
	 * <br>3 - Marc is joined to a Mixer (that is processing a conference):
	 * <pre>
	 *                ____________
	 *    _____      | MediaGroup |        
	 *   |     |---->|____________|
	 *   | Marc|                       _______
	 *   |_____|<-------------------->| Mixer |
	 *                                |_______|
	 * </pre>
	 * <br>4 - The application calls {@link SignalDetector#receiveSignals(int, javax.media.mscontrol.Parameter[], javax.media.mscontrol.resource.RTC[], Parameters) receiveSignals}
	 * on the SignalDetector of the MediaGroup.
	 * <p><code>receiveSignals</code> must complete with DTMF "1".
	 */
	public void test_2_1_12_8_BufferParticipantEarlyDTMF() throws Exception {
		playAndJoinAndFlush(false);
	}
	private void playAndJoinAndFlush(boolean doFlush) throws Exception {
		SipPhoneEmulator marc = getInbandSipPhoneEmulator("Marc");
		MediaGroup dtmfListenMG = myMediaSession.createMediaGroup(MediaGroup.SIGNALDETECTOR);
		marc.myJoinPoint.join(Direction.DUPLEX, dtmfListenMG);
		MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		
		marc.playPrompt(getProperty("testcase.dtmf1"));

		marc.myJoinPoint.join(Direction.DUPLEX, myMixer);
		SignalDetector dtmfListener = dtmfListenMG.getSignalDetector();
		EventWaiter<SignalDetectorEvent> listenEW = new EventWaiter<SignalDetectorEvent>(dtmfListener, log);
		if (doFlush) {
			dtmfListener.flushBuffer();
			listenEW.waitForOKEvent(3000); // wait for flushBuffer completion
		}
		dtmfListener.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		if (doFlush) {
			listenEW.failIfEvent(5000);
		} else {
			SignalDetectorEvent sdEvt = listenEW.waitForOKEvent(10000);
			assertEquals("Bad DTMF received", "1", sdEvt.getSignalString());
		}
	}
	/**
	 * Test that DTMFs emitted by conference participants, before they enter the conference, can be flushed.
	 * <p>Same as {@link #test_2_1_12_8_BufferParticipantEarlyDTMF()}, with an additional
	 * {@link SignalDetector#flushBuffer() flushBuffer} before step 4.
	 */
	public void test_2_1_12_9_FlushParticipantEarlyDTMF() throws Exception {
		playAndJoinAndFlush(true);		
	}
}
