/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mediagroup.signals;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;
import com.hp.opencall.jmsc.test.PatternChecker;

/**
 * Explore the most usual features of the SignalDetector.
 * <br>They include catching one or a few DTMFs known in advance
 * (like menu entry keys), catching a number of unknown DTMFs (like a PIN
 * number), and catching a {PIN or terminator}.
 * <p>All testcases use a {@link com.hp.opencall.jmsc.test.PatternChecker PatternChecker}.
 */
public class SignalDetectorTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(SignalDetectorTest.class);

	public SignalDetectorTest() {
		super();
	}
	public SignalDetectorTest(String testMethod) {
		super(testMethod);
	}
	/**
	 * Basics: check that we can detect one DTMF, no questions asked.
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector::1", "javadoc::SignalDetectorEvent::5" }, desc = "The SignalDetector can detect single DTMF signals")
	public void test_2_1_3_1_ReceiveOneSignal() throws Exception {
		PatternChecker checker = new PatternChecker(this, log);
		checker.expected = "1";
		checker.numSignals = 1;
		checker.run();
	}
	/**
	 * Basics: check that we can detect five DTMFs in a row.
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector::3" }, desc = "The SignalDetector can collect a sequence of signals into a buffer")
	public void test_2_1_3_2_ReceiveFiveSignals() throws Exception {
		PatternChecker checker = new PatternChecker(this, log);
		checker.expected = "12345";
		checker.numSignals = 5;
		checker.run();
	}

	/**
	 * Test with pattern0="123" defined by MG.setParameters().
	 * <br>The result must be "pattern0 matched", and the retrieved DTMFs must be "123".
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector::4", "javadoc::SignalDetectorEvent::9" }, desc = "The SignalDetector can match or detect a pattern in the signal buffer")
	public void test_2_1_3_3_Receive123() throws Exception {
		PatternChecker checker = new PatternChecker(this, log);
		checker.expected = "123";
		checker.patternStrings = new String[] {"123"};
		checker.matchingPatternIndex = 0;
		checker.run();
	}
	/**
	 * Similar to {@link #test_2_1_3_3_Receive123()}, with also pattern1="456".
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector::5" }, desc = "The SignalDetector can match or detect multiple patterns")
	public void test_2_1_3_4_Receive123And456() throws Exception {
		PatternChecker checker = new PatternChecker(this, log);
		checker.expected = "123";
		checker.patternStrings = new String[] {"123", "456"};
		checker.matchingPatternIndex = 0;
		checker.run();
	}

	/**
	 * Similar to {@link #test_2_1_3_3_Receive123()}, with:
	 * <br>- pattern0="45"
	 * <br>- pattern1="9"
	 * <br>- pattern2="123".
	 * Result must be "pattern2 matched", getSignalString="123"
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector::6", "javadoc::SignalDetector::20", "javadoc::SignalDetectorEvent::3" }, desc = "SD can match or detect multiple patterns in any order")
	public void test_2_1_3_5_Receive45And9And123() throws Exception {
		PatternChecker checker = new PatternChecker(this, log);
		checker.expected = "123";
		checker.patternStrings = new String[] {"45", "9", "123"};
		checker.matchingPatternIndex = 2;
		checker.run();
	}

	/**
	 * Define numSignals=4, pattern0="#".
	 * <br>Must return "1234".
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector::7" }, desc = "SD can combine numdigits and pattern")
	public void test_2_1_3_10_ReceivePoundOr4Digits() throws Exception {
		PatternChecker checker = new PatternChecker(this, log);
		checker.expected = "1234";
		checker.numSignals = 4;
		checker.patternStrings = new String[] {"#"};
		checker.matchingPatternIndex = -1;
		checker.run();
	}

	/**
	 * Define numSignals=4, pattern0="3", FILTERED_PATTERNS={pattern0}.
	 * <br>Must return "12": pattern0 stops the collection, and the
	 * last "3" is removed because of FILTERED_PATTERNS.
	 * <br>( pattern0 works as a terminator, like "#" in usual applications)
	 */
	@AssertionIds( ids = {"javadoc::SignalDetector::9" }, desc = "SD can remove signals from the signal buffer when a pattern is matched")
	public void test_2_1_3_11_Receive4DigitsOrDTMF3() throws Exception {
		PatternChecker checker = new PatternChecker(this, log);
		checker.expected = "12";
		checker.numSignals = 4;
		checker.patternStrings = new String[] {"3"};
		checker.filtered = new Boolean[]{true};
		checker.matchingPatternIndex = 0;
		checker.run();
	}
	
}
