/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mixer;

import java.net.URI;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.mediagroup.Recorder;
import javax.media.mscontrol.mediagroup.RecorderEvent;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.mediagroup.signals.SignalDetectorEvent;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.mixer.MixerAdapter;
import javax.media.mscontrol.resource.RTC;
import javax.media.mscontrol.resource.common.VolumeConstants;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;
import com.hp.opencall.jmsc.test.SipPhoneEmulator;

/**
 * 
 * Test the features usually embedded in a MixerAdapter: DTMF clamping and volume control.
 */
public class MixerAdapterTest extends MediaSessionBase {

	private static Logger log = Logger.getLogger(MixerAdapterTest.class);
	public MixerAdapterTest() {
		super();
	}
	public MixerAdapterTest(String testMethod) {
		super(testMethod);
	}
	
	
	/**
	 * Test the DTMF clamping feature of the MixerAdapter.
	 * <p>Two Media Groups are connected to a Mixer via looped NetworkConnections.
	 * One MG's player plays DTMF tone into the conference, the tone should be 
	 * detected by the other MG's Signal detector.  
	 * 
	 * Please refer to 
	 * {@link com.hp.opencall.jmsc.test.SipPhoneEmulator SipPhoneEmulator}
	 * for detailed description about the SipPhoneEmulator which contains
	 * a MG and 1 pair of looped NCs.
	 */
	@AssertionIds( ids = {"javadoc::MediaMixer::1", "javadoc::MixerAdapter::1", "javadoc::MixerAdapter::2" }, desc = "createMixerAdapter return a MixerAdapter")
	public void test_2_1_8_1_DTMFClamp() throws Exception {
		
		// Signal detector Phone
		SipPhoneEmulator myPhoneD = getInbandSipPhoneEmulator("detector");
		SignalDetector dtmfCatcher = myPhoneD.myMG.getSignalDetector();
		EventWaiter<SignalDetectorEvent> sdW = new EventWaiter<SignalDetectorEvent>(dtmfCatcher, log);
		// Player Phone 
		SipPhoneEmulator myPhoneP = getInbandSipPhoneEmulator("player");

		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		MixerAdapter adapter = myMixer.createMixerAdapter(MixerAdapter.EMPTY);
		adapter.join(Direction.DUPLEX, myPhoneP.myJoinPoint);
		myMixer.join(Direction.DUPLEX, myPhoneD.myJoinPoint);
		
		// Play and detect DTMF1
		dtmfCatcher.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		myPhoneP.playPrompt(getProperty("testcase.dtmf1"));
		SignalDetectorEvent sdE = sdW.waitForOKEvent(3000);
		log.info("Received DTMF: "+sdE.getSignalString());
		assertEquals("Bad DTMF received: ", "1", sdE.getSignalString());

		// 2nd Run, Player is joined to a MixAdapter with DTMF Clamp. 
		myPhoneP.myJoinPoint.unjoin(adapter);
		adapter = myMixer.createMixerAdapter(MixerAdapter.DTMF_CLAMP);
		adapter.join(Direction.DUPLEX, myPhoneP.myJoinPoint);
		dtmfCatcher.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		myPhoneP.playPrompt(getProperty("testcase.dtmf1"));
		// At this point we should NOT receive a SignalDetectorEvent
		sdW.failIfEvent(5000);
		myMixer.release();
	}
    /**
     * Test volume control via MixerAdapter.
     * <p>
     * A MediaGroup and a {@link SipPhoneEmulator} are joined to a Mixer.
     * <br>
     * The SipPhoneEmulator plays a file into the Mixer, and the MediaGroup records the Mixer output.
     * <br>A prompt file is played twice. In the first run, the volume (or gain) is increased every second while in the 
     * second run the volume drops every second. Listen to the recorded file to check the volume control.
     * @throws Exception
     */
	@AssertionIds( ids = {"javadoc::MixerAdapter::3" }, desc = "Configuration DTMFCLAMP_VOLUME allows volume control")
	public void test_2_1_8_2_VolumeControl() throws Exception {
		
		MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		MixerAdapter adapter = myMixer.createMixerAdapter(MixerAdapter.DTMFCLAMP_VOLUME);

		// Player 
		SipPhoneEmulator myPlayerPhone = getInbandSipPhoneEmulator("Player");
		adapter.join(Direction.DUPLEX, myPlayerPhone.myJoinPoint);
		Player myPlayer = myPlayerPhone.myMG.getPlayer();
		EventWaiter<PlayerEvent> playerEW = new EventWaiter<PlayerEvent>(myPlayer, log);

		// Recorder
		MediaGroup recMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		myMixer.join(Direction.DUPLEX, recMG);
		Recorder myRecorder = recMG.getRecorder();
		EventWaiter<RecorderEvent> recEW = new EventWaiter<RecorderEvent>(myRecorder, log);
		
		// Start recording
		myRecorder.record(URI.create(getProperty("testcase.mixeradapter.increasingvolume")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		// Play the prompt
		myPlayer.play(URI.create(getProperty("testcase.promptfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		
		for( int i = 0; i<5 ; i++) {
			playerEW.failIfEvent(1000);
			adapter.triggerAction(VolumeConstants.VOLUME_UP);
		}
		
		myRecorder.stop();
		recEW.waitForOKEvent(2000);
		myPlayer.stop(true);
		playerEW.waitForOKEvent(2000);
		
		// Start recording
		myRecorder.record(URI.create(getProperty("testcase.mixeradapter.decreasingvolume")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		// Play the prompt
		myPlayer.play(URI.create(getProperty("testcase.promptfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		
		for( int i = 0; i<5 ; i++) {
			playerEW.failIfEvent(1000);
			adapter.triggerAction(VolumeConstants.VOLUME_DOWN);
		}
		log.debug("Please listen to the files "+getProperty("testcase.mixeradapter.increasingvolume")+
				"and "+getProperty("testcase.mixeradapter.decreasingvolume")+" to check the volume control.");
	}
}
