/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mixer;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mixer.MediaMixer;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.ContinuityChecker;
import com.hp.opencall.jmsc.test.MediaSessionBase;

/**
 * Testcases joining a {@link MediaMixer} to one or multiple 
 * {@link javax.media.mscontrol.networkconnection.NetworkConnection NetworkConnections}.
 * <br>Varying the number of objects, the join Direction, unjoin, re-join, etc.
 * <br>All test methods use a {@link ContinuityChecker}, with various configurations.
 */
public class MixerJoinTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(MixerJoinTest.class);
	
	public MixerJoinTest() {
		super();
	}
	public MixerJoinTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * Bridge two calls through a mixer.
	 * <br>Create a MediaMixer (no parameters) and two {@link com.hp.opencall.jmsc.test.SipPhoneEmulator SipPhoneEmulator}.
	 * <br>Joins both phones in DUPLEX.
	 * <br>Play on one phone a file containing a DTMF, and verify
	 * that the DTMF is received on the other phone.
	 */
	public void test_2_1_6_1_BasicBridge() throws Exception {
		ContinuityChecker cc = getContinuityChecker(2, false);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		
		if (isTrue("manual-mode"))
			promptUser("Mixer is created, hit any key to continue: ", true);
		
		cc.getSender().join(Direction.DUPLEX, myMixer);
		
		if (isTrue("manual-mode"))
			promptUser("First participant is joined, hit any key to continue: ", true);
		
		cc.getReceiver(0).join(Direction.DUPLEX, myMixer);
		
		if (isTrue("manual-mode"))
			promptUser("Second participant is joined, hit any key to continue: ", true);
		
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(0, new Boolean[]{false, true}));
	}
	
	/**
	 * Connect 3 {@link com.hp.opencall.jmsc.test.SipPhoneEmulator SipPhoneEmulator} on a Mixer.
	 * <br>Create a MediaMixer (no parameters) and 3 {@link com.hp.opencall.jmsc.test.SipPhoneEmulator SipPhoneEmulator}.
	 * <br>Joins all phones in DUPLEX.
	 * <br>Play on one phone a file containing a DTMF, and verify
	 * that the DTMF is received on the 2 other phones.
	 */
	public void test_2_1_6_2_ThreeWaysBridge() throws Exception {
		ContinuityChecker cc = getContinuityChecker(3, false);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		cc.getSender().join(Direction.DUPLEX, myMixer);
		cc.getReceiver(0).join(Direction.DUPLEX, myMixer);
		cc.getReceiver(1).join(Direction.DUPLEX, myMixer);
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(0, new Boolean[]{false, true, true}));
	}
	/**
	 * Same as {@link #test_2_1_6_2_ThreeWaysBridge()} with 1+5 phones.
	 * <br>All phones are joined in DUPLEX to the single Mixer.
	 * <br>Each SipPhoneEmulator is tested as a sender, the other being receivers
	 */
	public void test_2_1_6_3_FiveWaysSplitter() throws Exception {
		ContinuityChecker cc = getContinuityChecker(6, false);
		Parameters mixerOpts = createParameters();
		mixerOpts.put(MediaMixer.MAX_PORTS, 6);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO, mixerOpts);
		cc.getSender().join(Direction.DUPLEX, myMixer);
		cc.getReceiver(0).join(Direction.DUPLEX, myMixer);
		cc.getReceiver(1).join(Direction.DUPLEX, myMixer);
		cc.getReceiver(2).join(Direction.DUPLEX, myMixer);
		cc.getReceiver(3).join(Direction.DUPLEX, myMixer);
		cc.getReceiver(4).join(Direction.DUPLEX, myMixer);
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(0, new Boolean[]{false, true, true, true, true, true}));
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(1, new Boolean[]{true, false, true, true, true, true}));
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(2, new Boolean[]{true, true, false, true, true, true}));
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(3, new Boolean[]{true, true, true, false, true, true}));
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(4, new Boolean[]{true, true, true, true, false, true}));
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(5, new Boolean[]{true, true, true, true, true, false}));
	}
	/**
	 * Check that unjoin actually blocks media stream.
	 * <br>Create a MediaMixer (no parameters) and two {@link com.hp.opencall.jmsc.test.SipPhoneEmulator SipPhoneEmulator}, one sender
	 * and one receiver.
	 * <br>Joins both phones in DUPLEX on the MediaMixer.
	 * <br>Unjoin the receiver phone
	 * <br>Play on one phone a file containing a DTMF, and verify
	 * that the DTMF is <b>not</b> received on the other phone.
	 * <pre>
	 * 1) sender <--> Mixer
	 * 2) sender <--> Mixer <--> receiver
	 * 3) sender <--> Mixer      receiver
	 * </pre>
	 * Operations:
	 * <br>sender.join(DUPLEX, Mixer);
	 * <br>receiver.join(DUPLEX, Mixer);
	 * <br>receiver.unjoin(Mixer);
	 */
	public void test_2_1_6_4_UnjoinedBridge() throws Exception {
		ContinuityChecker cc = getContinuityChecker(2, false);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		cc.getSender().join(Direction.DUPLEX, myMixer);
		cc.getReceiver(0).join(Direction.DUPLEX, myMixer);
		cc.getReceiver(0).unjoin(myMixer);
		assertTrue("DTMFs still go through SipPhoneEmulators", cc.isContinuous(0, new Boolean[]{false, false}));
	}
	/**
	 * Check that joining in Direction.RECV actually blocks the mediastream in one direction.
	 * <br>Create a MediaMixer (no parameters) and two {@link com.hp.opencall.jmsc.test.SipPhoneEmulator SipPhoneEmulator},
	 * one sender and one receiver.
	 * <br>Joins the sender in DUPLEX and the receiver in RECV.
	 * <br>Play on the sender a file containing a DTMF, and verify
	 * that the DTMF is received on the receiver phone.
	 * <pre>
	 * 1) sender <--> Mixer
	 * 2) sender <--> Mixer  --> receiver
	 * </pre>
	 * Operations:
	 * <br>sender.join(DUPLEX, Mixer);
	 * <br>receiver.join(RECV, Mixer);
	 * <p>The media must go through.
	 */
	public void test_2_1_6_8_BasicBridgeRecvOK() throws Exception {
		ContinuityChecker cc = getContinuityChecker(2, false);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		cc.getSender().join(Direction.DUPLEX, myMixer);
		cc.getReceiver(0).join(Direction.RECV, myMixer);
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(0, new Boolean[]{false, true}));
	}
	/**
	 * Same as {@link #test_2_1_6_8_BasicBridgeRecvOK()}, with sender in RECV.
	 * <pre>
	 * 1) sender <-- Mixer
	 * 2) sender <-- Mixer <--> receiver
	 * </pre>.
	 * Operations:
	 * <br>sender.join(RECV, Mixer)
	 * <br>receiver.join(DUPLEX, Mixer)
	 * <p>The media must <b>not</b> go through.
	 */
	public void test_2_1_6_9_BasicBridgeRecvNOK() throws Exception {
		ContinuityChecker cc = getContinuityChecker(2, false);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		cc.getSender().join(Direction.RECV, myMixer);
		cc.getReceiver(0).join(Direction.DUPLEX, myMixer);
		assertTrue("DTMFs still go through SipPhoneEmulators", cc.isContinuous(0, new Boolean[]{false, false}));
	}
	/**
	 * Same as {@link #test_2_1_6_9_BasicBridgeRecvNOK()}, but the sender is joined DUPLEX first,
	 * and then RECV.
	 * <pre>
	 * 1) sender <--> Mixer
	 * 2) sender <--  Mixer
	 * 3) sender <--  Mixer <--> receiver
	 * </pre>
	 * Operations:
	 * <br>sender.join(DUPLEX, Mixer);
	 * <br>sender.join(RECV, Mixer);
	 * <br>receiver.join(DUPLEX, Mixer);
	 * <p>The media must <b>not</b> go through.
	 */
	public void test_2_1_6_10_BasicBridgeRejoinRecvNOK() throws Exception {
		ContinuityChecker cc = getContinuityChecker(2, false);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		cc.getSender().join(Direction.DUPLEX, myMixer);
		cc.getSender().join(Direction.RECV, myMixer);
		cc.getReceiver(0).join(Direction.DUPLEX, myMixer);
		assertTrue("DTMFs still go through SipPhoneEmulators", cc.isContinuous(0, new Boolean[]{false, false}));
	}
	/**
	 * Check that after being muted (RECV), the sender can be set on normal again (DUPLEX) and send media.
	 * <pre>
	 * 1) sender <--  Mixer
	 * 2) sender <--  Mixer <--> receiver
	 * 3) sender <--> Mixer <--> receiver
	 * </pre>
	 */
	public void test_2_1_6_11_BasicBridgeUnMute() throws Exception {
		ContinuityChecker cc = getContinuityChecker(2, false);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		cc.getSender().join(Direction.RECV, myMixer);
		cc.getReceiver(0).join(Direction.DUPLEX, myMixer);
		cc.getSender().join(Direction.DUPLEX, myMixer);
		assertTrue("DTMFs do not go through SipPhoneEmulators", cc.isContinuous(0, new Boolean[]{false, true}));
	}		
}
