/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mixer;

import java.net.URI;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.mediagroup.signals.SignalDetectorEvent;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.ContinuityChecker;
import com.hp.opencall.jmsc.test.MediaSessionBase;
import com.hp.opencall.jmsc.test.SipPhoneEmulator;

/**
 * Tests the various cases of join/rejoin, send/recv, involving
 * a {@link MediaMixer} and a {@link MediaGroup}.
 * <p>Testing requires also one of more {@link javax.media.mscontrol.networkconnection.NetworkConnection NetworkConnections}
 * <br>This is the typical use case for playing an announcement into, or recording a conference.
 */
public class MixerMediaGroupTest extends MediaSessionBase {
	
	Logger log = Logger.getLogger(MixerMediaGroupTest.class);
	
	public MixerMediaGroupTest() {
		super();
	}
	public MixerMediaGroupTest(String testMethod) {
		super(testMethod);
	}
	
	/**
	 * Join a SipPhoneEmulator (named "Paul") and a MediaGroup on a Mixer, DUPLEX.
	 * <br>
	 * Resulting graph:
	 * <pre>
	 *               _______
	 *              |       | 
	 *    Paul <--> | Mixer | <--> MediaGroup
	 *              |_______| 
	 * </pre>
	 * Paul sends the DTMF "1", which must be caught by the SignalDetector of the MediaGroup.
	 * <br>
	 * @throws Exception
	 */
	@AssertionIds( ids = { "javadoc::MediaGroup::3", "javadoc::MediaGroup::9" }, desc = ", getSignalDetector")
	public void test_2_1_13_1_MGJoinMixer() throws Exception {
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		final MediaGroup anna = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		final SipPhoneEmulator myPhone = getInbandSipPhoneEmulator("Paul");
		final MediaGroup paul = myPhone.myMG;
		myMixer.join(Direction.DUPLEX, anna);
		myMixer.join(Direction.DUPLEX, myPhone.myJoinPoint);
		final SignalDetector sigDet = anna.getSignalDetector();
		EventWaiter<SignalDetectorEvent> sdW = new EventWaiter<SignalDetectorEvent>(sigDet, log);
		sigDet.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		paul.getPlayer().play(URI.create(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sdE = sdW.waitForOKEvent(20000);
		assertEquals("Bad DTMF received: ","1", sdE.getSignalString());
		log.info("test_2_1_13_1_MGJoinMixer completed");
	}
	/**
	 * Check that media flows through a Mixer, using one MediaGroup and
	 * three {@link com.hp.opencall.jmsc.test.SipPhoneEmulator SipPhoneEmulator}'s.
	 * <p>
	 * Resulting graph:
	 * <pre>
	 *                   _______
	 *                  |       | <--> receiver0
	 *                  |       | 
	 *  MediaGroup <--> | Mixer | 
	 *                  |       |     
	 *                  |_______|  --> receiver1
	 * </pre>
	 * 3 tests are performed, with a different sender each time.
	 * 
	 */
	public void test_2_1_13_2_MGJoinMixerTwoParticipants() throws Exception {
		ContinuityChecker cc = getContinuityChecker(3, true);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		myMixer.join(Direction.DUPLEX, cc.getSender());
		myMixer.join(Direction.DUPLEX, cc.getReceiver(0));
		myMixer.join(Direction.SEND, cc.getReceiver(1));
		assertTrue("DTMFs reception error", cc.isContinuous(0, new Boolean[]{false, true, true}));
		assertTrue("DTMFs reception error", cc.isContinuous(1, new Boolean[]{true, false, true}));
		assertTrue("DTMFs reception error", cc.isContinuous(2, new Boolean[]{false, false, false}));
	}
}
