/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mixer;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.mediagroup.signals.SignalDetectorEvent;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.MediaSessionBase;
import com.hp.opencall.jmsc.test.SipPhoneEmulator;

/**
 * Test the DTMF detection from a {@link javax.media.mscontrol.networkconnection.NetworkConnection NetworkConnection},
 * while it is also joined/unjoined in send/recv to a {@link MediaMixer}.
 * <br>This is the typical use-case when monitoring DTMFs from a conference participant.
 */
public class ParticipantDTMFTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(ParticipantDTMFTest.class);
	
	public ParticipantDTMFTest() {
		super();
	}
	public ParticipantDTMFTest(String testMethod) {
		super(testMethod);
	}
	
	/**
	 * A MediaGroup joined to a NetworkConnection catches DTMFs while the NC is joined to a Mixer.
	 * <ol>
	 * <li>MG <--> NC</li>
	 * <li>MG <--  NC <--> Mixer</li>
	 * <li>MG.receiveSignals</li>
	 * </ol>
	 * After the setup as described above, where the NC is the joint point of a
	 * {@link com.hp.opencall.jmsc.test.SipPhoneEmulator SipPhoneEmulator}, the emulator
	 * sends a DTMF "1", that must be caught by the MediaGroup's SignalDetector.
	 */
	public void test_2_1_15_1_CatchDTMFOnParticipant() throws Exception {
		SipPhoneEmulator myPhone = getInbandSipPhoneEmulator("Paul");
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		myMG.join(Direction.DUPLEX, myPhone.myJoinPoint);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		myMixer.join(Direction.DUPLEX, myPhone.myJoinPoint);
		SignalDetector dtmfCatcher = myMG.getSignalDetector();
		EventWaiter<SignalDetectorEvent> sdW = new EventWaiter<SignalDetectorEvent>(dtmfCatcher, log);
		dtmfCatcher.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		myPhone.playPrompt(getProperty("testcase.dtmf1"));
		SignalDetectorEvent sdE = sdW.waitForOKEvent(3000);
		log.info("Received DTMF: "+sdE.getSignalString());
		assertEquals("Bad DTMF received: ", "1", sdE.getSignalString());
	}
	/**
	 * Verify that a {@link SignalDetector#receiveSignals(int, javax.media.mscontrol.Parameter[], javax.media.mscontrol.resource.RTC[], javax.media.mscontrol.Parameters) receiveSignals}
	 * can be running while the NetworkConnection is rejoined RECV to the MediaMixer.
	 * <p>
	 * <br>Setup:
	 * <pre>
	 *                        _______________
	 *    ____________       |  MediaGroup   |        
	 *   |            | ---->|               |
	 *   |   Paul     |      |_______________|
	 *   |            |                       _______ 
	 *   |____________|<--------------------?| Mixer |
	 *                                       |       |
	 *                                       |_______|
	 * </pre>
	 * The question-mark indicates the relationship that is changing during the test.
	 */
	public void test_2_1_15_2_MuteAndDTMF()throws Exception  {
		SipPhoneEmulator myPhone = getInbandSipPhoneEmulator("Paul");
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		myPhone.myJoinPoint.join(Direction.DUPLEX, myMG);
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		myPhone.myJoinPoint.join(Direction.DUPLEX, myMixer);
		SignalDetector dtmfCatcher = myMG.getSignalDetector();
		EventWaiter<SignalDetectorEvent> sdW = new EventWaiter<SignalDetectorEvent>(dtmfCatcher, log);
		dtmfCatcher.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		sdW.failIfEvent(1000);
		myPhone.myJoinPoint.join(Direction.RECV, myMixer);
		sdW.failIfEvent(1000);
		myPhone.myJoinPoint.join(Direction.DUPLEX, myMixer);
		myPhone.myJoinPoint.join(Direction.RECV, myMixer);		
		myPhone.playPrompt(getProperty("testcase.dtmf1"));
		SignalDetectorEvent sdE = sdW.waitForOKEvent(3000);
		log.info("Received DTMF: "+sdE.getSignalString());
		assertEquals("Bad DTMF received: ", "1", sdE.getSignalString());		
	}
}
