/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.mixer;

import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mixer.MediaMixer;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.ContinuityChecker;
import com.hp.opencall.jmsc.test.MediaSessionBase;
/**
 * Tests the various cases of join/rejoin, send/recv, involving
 * a {@link MediaMixer}, a {@link javax.media.mscontrol.networkconnection.NetworkConnection NetworkConnection}
 * and a {@link MediaGroup}.
 * <br>This is the typical use case for a conference participant, where the MediaGroup is monitoring
 * the participant's DTMFs.
 */
public class ParticipantJoinTest extends MediaSessionBase {
	
	Logger log = Logger.getLogger(ParticipantJoinTest.class);
	
	public ParticipantJoinTest() {
		super();
	}
	public ParticipantJoinTest(String testMethod) {
		super(testMethod);
	}
	
	/**
	 * Test that a {@link javax.media.mscontrol.networkconnection.NetworkConnection NetworkConnection}
	 * can be joined DUPLEX alternatively to a {@link MediaGroup} and a {@link MediaMixer}.
	 * <p>The NetworkConnection should be able to SEND media to both objects at all times.
	 * <p>Uses a 3-ways {@link ContinuityChecker} (where the 1st pin is a MediaGroup), and a Mixer
	 * <br>Setup:
	 * <pre>
	 *                        _______________
	 *    ____________       |  MediaGroup   |        
	 *   |            |?---->| CChecker-MG-0 |
	 *   |   Paul     |      |_______________|
	 *   | CChecker-1 |                       _______        ____________
	 *   |____________|?-------------------->| Mixer |      |   Marc     |
	 *                                       |       |<---->| CChecker-2 |
	 *                                       |_______|      |____________|
	 * </pre>
	 * The testcase has 5 steps, alternating join DUPLEX to the MediaGroup and the MediaMixer,
	 * and playing announcements to Paul.
	 */
	public void test_2_1_14_1_EnterAndLeaveConference() throws Exception {
		ContinuityChecker cc = getContinuityChecker(3, true);
		
		// 1 - join Paul to MediaGroup
		cc.getPin(1).join(Direction.DUPLEX, cc.getPin(0));
		assertTrue("MediaGroup does not get DTMFs from Paul", cc.isContinuous(1, new Boolean[]{true, false, false}));

		// 2 - join Paul to Mixer (and Marc as well)
		log.info("test_2_1_14_1_EnterAndLeaveConference - step 2");
		MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		cc.getPin(2).join(Direction.DUPLEX, myMixer);
		cc.getPin(1).join(Direction.DUPLEX, myMixer);
		assertTrue("MediaGroup or Marc does not get DTMFs from Paul", cc.isContinuous(1, new Boolean[]{true, false, true}));
		assertTrue("Paul does not get DTMFs from Marc", cc.isContinuous(2, new Boolean[]{false, true, false}));
		
		// 3 - join Paul to MediaGroup again
		log.info("test_2_1_14_1_EnterAndLeaveConference - step 3");
		cc.getPin(1).join(Direction.DUPLEX, cc.getPin(0));
		// At this stage, the join between NC and Mixer should be degraded in SEND.
		// However, "join in SEND to MediaMixer" is an optional feature.
		// Some implementations may keep DUPLEX, some others may go "parked".
		// Depending on the implementation choice, one of the asserts below would fail.
		// For this reason the asserts are removed from this mandatory testcase.
		assertTrue("MediaGroup or Marc does not get DTMFs from Paul", cc.isContinuous(1, new Boolean[]{true, false, true}));
		assertTrue("Paul does get DTMFs from Marc", cc.isContinuous(2, new Boolean[]{false, false, false}));
		
		// 4 - Play an annoucement to Paul, using the MediaGroup
		//     The announcement is actually the DTMF sent by the ContinuityChecker ...
		log.info("test_2_1_14_1_EnterAndLeaveConference - step 4");
		assertTrue("Paul does not get DTMFs from the MediaGroup", cc.isContinuous(0, new Boolean[]{false, true, false}));
		
		// 5 - join Paul to Mixer again
		log.info("test_2_1_14_1_EnterAndLeaveConference - step 5");
		cc.getPin(1).join(Direction.DUPLEX, myMixer);
		assertTrue("MediaGroup or Marc does not get DTMFs from Paul", cc.isContinuous(1, new Boolean[]{true, false, true}));
		assertTrue("Paul does not get DTMFs from Marc", cc.isContinuous(2, new Boolean[]{false, true, false}));
	}
}
