/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.networkconnection;

import javax.media.mscontrol.networkconnection.CodecPolicy;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.networkconnection.SdpException;
import javax.media.mscontrol.networkconnection.SdpPortManager;
import javax.media.mscontrol.networkconnection.SdpPortManagerEvent;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;

/**
 * Test the functionalities of a {@link CodecPolicy}.
 * <p>All test cases proceed as follows:
 * <ul>
 * <li>Select one of two arbitrarily built SDP offers.
 * One is a rich offer with many codecs, the other one is minimal with only PCMU.
 * Both are audio-only, and defined by static Strings in this class: {@link #minimalOffer}, {@link #richOffer}.
 * </li>
 * <li>Prepare a CodecPolicy with the tested characteritics.
 * </li>
 * <li>Create a {@link NetworkConnection}, set the CodecPolicy to its {@link SdpPortManager},
 * and call {@link SdpPortManager#processSdpOffer(byte[])} with the selected SDP.
 * </li>
 * <li>Run assertions on the following: Exception or not, completion event with success or not, content of SDP answer.
 * <br>See {@link #applyPolicyToOffer(String, CodecPolicy, boolean)} for details.
 * </li> 
 * </ul>
 * The expected setup failures are caused by
 * <ul>
 * <li>no remaining media-type</li>
 * <li>no remaining codec</li>
 * <li>missing required codec</li>
 * </ul>
 * <p>The media server must support at least PCMU, PCMA, AMR and telephone-event.
 * <p>Note: the fact that the preferences are reflected in the SDP answer is not tested.
 * The media server may decide to use a different codec, and this is not controllable
 * by the testcase.
 */
public class CodecPolicyTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(CodecPolicyTest.class);

	public CodecPolicyTest() {
		super();
	}
	public CodecPolicyTest(String method) {
		super(method);
	}

	/**
	 * An example of valid SDP offer, including many codecs.
	 */
	public static final String richOffer = 
		"v=0\r\n"+
		"o=HP-OCMP 756 2 IN IP4 16.16.93.241\r\n"+
		"s=-\r\n"+
		"c=IN IP4 16.16.93.241\r\n"+
		"t=0 0\r\n"+
		"m=audio 42412 RTP/AVP 0 8 4 18 97 99 101 100\r\n"+
		"c=IN IP4 16.16.93.241\r\n"+
		"b=TIAS:64000\r\n"+
		"b=AS:80\r\n"+
		"a=rtpmap:0 PCMU/8000/1\r\n"+
		"a=rtpmap:8 PCMA/8000/1\r\n"+
		"a=rtpmap:4 G723/8000/1\r\n"+
		"a=fmtp:4 bitrate=5.3,6.3;annexb=yes\r\n"+
		"a=rtpmap:18 G729/8000/1\r\n"+
		"a=fmtp:18 annexb=yes\r\n"+
		"a=rtpmap:97 EVRC0/8000/1\r\n"+
		"a=rtpmap:99 AMR/8000/1\r\n"+
		"a=fmtp:99 mode-set=7;octet-align=1\r\n"+
		"a=rtpmap:101 telephone-event/8000/1\r\n"+
		"a=rtpmap:100 AMR-WB/16000/1\r\n"+
		"a=fmtp:100 mode-set=8;octet-align=1\r\n"+
		"a=sendrecv\r\n";

	/**
	 * An example of (valid) single-codec offer, not even including the rtpmap.
	 */
	public static final String minimalOffer = 
		"v=0\r\n"+
		"o=HP-OCMP 756 2 IN IP4 16.16.93.241\r\n"+
		"s=-\r\n"+
		"c=IN IP4 16.16.93.241\r\n"+
		"t=0 0\r\n"+
		"m=audio 42412 RTP/AVP 0\r\n";

	/**
	 * Make a call to {@link SdpPortManager#processSdpOffer(byte[])} with the given offer, after setting
	 * the given {@link CodecPolicy}.
	 * <br>This method is void but includes the required assertions to verifiy the result.
	 * <p>Three possibilities are handled:
	 * <ul>
	 * <li>An {@link SdpException} is thrown by processOffer. If <code>expectError</code>
	 * is true, this is acceptable and considered as a success completion, else the exception is
	 * not caught.
	 * </li>
	 * <li>The completion event contains an error. This MUST be true if <code>expectError</code> is true,
	 * else the test is terminated with an error.
	 * </li>
	 * <li>The completion event contains no error, and a valid SDP answer.
	 * This answer is checked to NOT contain any excluded codec, and DOES contain all required codecs.
	 * </li>
	 * </ul>
	 * @param offer The SDP offer, "initial offer" in the CodecPolicy documentation.
	 * @param policy The CodecPolicy to apply
	 * @param expectError true if the caller expects that the setup will fail.
	 * The error is checked by this method in this case.
	 */
	protected void applyPolicyToOffer(String offer, CodecPolicy policy, boolean expectError) throws Exception {
		// Show what we are applying
		showCodecPolicy(policy, log);
		
		// Create and setup the SdpPortManager
		SdpPortManager myPorts = myMediaSession.createNetworkConnection(NetworkConnection.BASIC).getSdpPortManager();
		myPorts.setCodecPolicy(policy);
		EventWaiter<SdpPortManagerEvent> waiter = new EventWaiter<SdpPortManagerEvent>(myPorts, log);
		if (expectError) {
			// MAY throw exception, or accept the call and return a MediaErr in the completion event.
			// The testcases accept both.
			try {
				myPorts.processSdpOffer(offer.getBytes());
			} catch (SdpException e) {
				log.info("Caught expected SdpException: "+e.getMessage());
				return;
			}
		} else {
			myPorts.processSdpOffer(offer.getBytes());
		}
		
		// Wait for the setup to complete
		SdpPortManagerEvent event = waiter.waitForAnyEvent(3000);
		if (expectError) {
			assertEquals("Bad event error: "+event.getError(), SdpPortManagerEvent.SDP_NOT_ACCEPTABLE, event.getError());
			return;
		}
		if (! event.isSuccessful())
			fail("Unexpected SdpPortManagerEvent error: "+event.getError()+", \""+event.getErrorText()+"\"");
		
		// The media server returned an SDP, let us check it
		checkSdpAnswer(new String(event.getMediaServerSdp()), policy, log);
	}
	/** 
	 * Front-end helper for calling {@link #applyPolicyToOffer(String, CodecPolicy, boolean)}
	 * with only excluded codecs.
	 * <br>The only added value is to enable single-line call (factorization).
	 */
	protected void excludeCodecs(String offer, String[] excluded, boolean expectError) throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setExcludedCodecs(excluded);
		applyPolicyToOffer(offer, myPolicy, expectError);
	}
	protected void requireCodecs(String offer, String[] required, boolean expectError) throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setRequiredCodecs(required);
		applyPolicyToOffer(offer, myPolicy, expectError);
	}
	protected void preferCodecs(String offer, String[] preferred) throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setCodecPreferences(preferred);
		applyPolicyToOffer(offer, myPolicy, false);
	}
	protected void capableCodecs(String offer, String[] capable, boolean expectError)  throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setCodecCapabilities(capable);
		applyPolicyToOffer(offer, myPolicy, expectError);
	}
	/**
	 * Exclude "telephone-event" from a rich SDP offer.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::2", "javadoc::SdpPortManager::7" }, desc = "telephone-event can be excluded with setExcludedCodecs")
	public void test_2_1_18_1_ExcludeTelephoneEvent() throws Exception {
		excludeCodecs(richOffer, new String[]{"telephone-event"}, false);
	}
	/**
	 * Exclude one of the multiple codecs of a rich SDP offer.
	 * <br>7 differents codecs are tested, one at a time: G723, G729, EVRC0, AMR, AMR-WB, PCMA, PCMA
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::11" }, desc = "Each major audio codec can be excluded with setExcludedCodecs")
	public void test_2_1_18_2_ExcludeOneCodec() throws Exception {
		excludeCodecs(richOffer, new String[]{"G723"}, false);
		excludeCodecs(richOffer, new String[]{"G729"}, false);
		excludeCodecs(richOffer, new String[]{"EVRC0"}, false);
		excludeCodecs(richOffer, new String[]{"AMR-WB"}, false);
		excludeCodecs(richOffer, new String[]{"AMR"}, false);
		excludeCodecs(richOffer, new String[]{"PCMA"}, false);
		excludeCodecs(richOffer, new String[]{"PCMU"}, false);
	}
	
	/**
	 * Exclude almost all codecs of a rich offer, keeping only one (AMR).
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::12" }, desc = "All major audio codecs can be excluded with setExcludedCodecs, keeping AMR")
	public void test_2_1_18_3_ExcludeMostCodecs() throws Exception {
		excludeCodecs(richOffer, new String[]{"G723","G729","EVRC0","AMR-WB",
				"telephone-event","PCMA","PCMU"}, false);
	}
	/**
	 * Exclude all codecs of a rich offer (Expect an error).
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::13" }, desc = "Excluding all codecs causes SDP_NOT_ACCEPTABLE or SdpException")
	public void test_2_1_18_4_ExcludeAllCodecs() throws Exception {
		excludeCodecs(richOffer, new String[]{"G723","AMR","G729","EVRC0",
					"AMR-WB","telephone-event","PCMA","PCMU"}, true);
	}
	/**
	 * Same as {@link #test_2_1_18_2_ExcludeOneCodec()}, but applied to a
	 * minimal SDP offer.
	 * <br>The last case excludes also the single codec of the offer, and should
	 * cause an error.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::14" }, desc = "setExcludedCodecs is no-op when codec is not offered")
	public void test_2_1_18_5_ExcludeCodecsMinimalOffer() throws Exception {
		excludeCodecs(minimalOffer, new String[]{"telephone-event"}, false);
		excludeCodecs(minimalOffer, new String[]{"G723"}, false);
		excludeCodecs(minimalOffer, new String[]{"AMR-WB"}, false);
		excludeCodecs(minimalOffer, new String[]{"PCMU"}, true);
		excludeCodecs(minimalOffer, new String[]{"G723","AMR","G729","EVRC0",
				"AMR-WB","telephone-event","PCMA","PCMU"}, true);
	}
	/**
	 * Require "telephone-event", which is present.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::6" }, desc = "telephone-event can be set as the required codec")
	public void test_2_1_18_6_RequireTelephoneEvent() throws Exception {
		requireCodecs(richOffer, new String[]{"telephone-event"}, false);
	}
	/**
	 * Require "PCMU" and "telephone-event", which are present in the offer.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::15" }, desc = "PCMU and telephone-event can be set as the required codecs")
	public void test_2_1_18_7_RequirePcmuTelephoneEvent() throws Exception {
		requireCodecs(richOffer, new String[]{"PCMU", "telephone-event"}, false);
	}
	/**
	 * Require "telephone-event" and "AMR", which are present.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::16" }, desc = "telephone-event and AMR can be set as the required codecs")
	public void test_2_1_18_8_RequireTelephoneEventAmr() throws Exception {
		requireCodecs(richOffer, new String[]{"telephone-event", "AMR"}, false);
	}
	/**
	 * Require an absent codec (expect error).
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::17" }, desc = "setting an absent codec as required causes SDP_NOT_ACCEPTABLE or SdpException")
	public void test_2_1_18_9_RequireAbsentCodec() throws Exception {
		requireCodecs(richOffer, new String[]{"G276"}, true);
	}
	/**
	 * Prefer "PCMU". The fact that is actually selected is not tested.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::8" }, desc = "setCodecPreferences indicates a preference between codecs")
	public void test_2_1_18_10_PreferPcmu() throws Exception {
		preferCodecs(richOffer, new String[]{"PCMU"});
	}
	/**
	 * Prefer "AMR". The fact that is actually selected is not tested.
	 */
	public void test_2_1_18_11_PreferAmr() throws Exception {
		preferCodecs(richOffer, new String[]{"AMR"});
	}
	/**
	 * Prefer "PCMU", "PCMA" and "telephone-event". The fact that they are actually selected is not tested.
	 */
	public void test_2_1_18_12_PreferPcmuPcmaTelephoneEvent() throws Exception {
		preferCodecs(richOffer, new String[]{"PCMU", "PCMA", "telephone-event"});
	}
	/**
	 * Set CodecCapabilities to "PCMU".
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::4" }, desc = "Setting CodecCapabilities to PCMU causes it to be retained")
	public void test_2_1_18_13_CapablePcmu() throws Exception {
		capableCodecs(richOffer, new String[]{"PCMU"}, false);
	}
	/**
	 * Set CodecCapabilities to "PCMU", "PCMA", "telephone-event", on a rich offer.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::18" }, desc = "Setting CodecCapabilities to PCMU/PCMA/telephone-event causes PCMU or PCMA to be retained")
	public void test_2_1_18_14_CapablePcmuPcmaTelephoneEvent() throws Exception {
		capableCodecs(richOffer, new String[]{"PCMU", "PCMA", "telephone-event"}, false);
	}
	/**
	 * Set CodecCapabilities to "G726" and "telephone-event", on an offer limited to "PCMU" (expect error).
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::19" }, desc = "When intersection of offer and CodecCapabilities is only telephone-event, error is detected")
	public void test_2_1_18_15_CapableG726TelephoneEvent() throws Exception {
		capableCodecs(minimalOffer, new String[]{"G726", "telephone-event"}, true);
	}
	/**
	 * Set CodecCapabilities to "G726" and "G722", on a rich offer that does not include them (expect error).
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::20" }, desc = "When intersection of offer and CodecCapabilities is null, error is detected")
	public void test_2_1_18_16_CapableG726G722() throws Exception {
		capableCodecs(richOffer, new String[]{"G726", "G722"}, true);
	}
	/**
	 * With a rich offer, set CodecCapabilities to "PCMU", "PCMA", "telephone-event", and "AMR", and
	 * preferences to "AMR".
	 * The fact that AMR is selected is not tested.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::21" }, desc = "A CodecPolicy can combine capabilities and preferences")
	public void test_2_1_18_17_CapablePcmuPcmaTeAmrPreferAmr() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setCodecCapabilities(new String[]{"PCMU", "PCMA", "telephone-event", "AMR"});
		myPolicy.setCodecPreferences(new String[]{"AMR"});
		applyPolicyToOffer(richOffer, myPolicy, false);		
	}
	/**
	 * Test a complex CodecPolicy, including Capabilities, Preferences, Required and Excluded codecs.
	 * <br>Uses a rich offer, able to satisfy all conditions.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::22" }, desc = "A CodecPolicy can combine all 4 requirement types")
	public void test_2_1_18_18_ComplexPolicy() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setCodecCapabilities(new String[]{"PCMU", "PCMA", "telephone-event", "AMR"});
		myPolicy.setCodecPreferences(new String[]{"AMR", "PCMU"});
		myPolicy.setExcludedCodecs(new String[]{"EVRC0", "G723"});
		myPolicy.setRequiredCodecs(new String[]{"telephone-event"});
		applyPolicyToOffer(richOffer, myPolicy, false);	
	}
	/**
	 * Set the MediaTypeCapability to "audio".
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::10" }, desc = "media-type capability can be restricted to audio")
	public void test_2_1_18_19_AudioCapability() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setMediaTypeCapabilities(new String[]{"audio"});
		applyPolicyToOffer(richOffer, myPolicy, false);			
	}
	/**
	 * Set the MediaTypeCapability to "video" and "audio".
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::23" }, desc = "media-type capability can be restricted to audio and video")
	public void test_2_1_18_20_AudioVideoCapability() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setMediaTypeCapabilities(new String[]{"video", "audio"});
		applyPolicyToOffer(richOffer, myPolicy, false);			
	}
	/**
	 * Set the MediaTypeCapability to "video", causing error.
	 */
	@AssertionIds( ids = {"javadoc::CodecPolicy::24" }, desc = "restricting media-types to an absent media-type causes error")
	public void test_2_1_18_21_VideoCapability() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setMediaTypeCapabilities(new String[]{"video"});
		applyPolicyToOffer(richOffer, myPolicy, true);			
	}
}
