/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.functional.networkconnection;

import java.net.URI;

import javax.media.mscontrol.MediaEventListener;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.mediagroup.Recorder;
import javax.media.mscontrol.mediagroup.RecorderEvent;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;
import com.hp.opencall.jmsc.test.mandatory.functional.mediagroup.PlayerTest;

/**
 * Verify that a NetworkConnection can be setup in the two main modes: with an offer provided
 * as argument (received from a user agent from example), or without an offer (as when performing
 * an outgoing call).
 * As an interesting usecase, verify that two NetworkConnections can be setup by exchanging their local and remote SDP.
 * <br>This is actually creating a loopback RTP stream on the media server, and is used as a tool by
 * many other testcases, see {@link com.hp.opencall.jmsc.test.SipPhoneEmulator} and
 * {@link com.hp.opencall.jmsc.test.LoopbackMediaGroup}.
 * <br><b>Note</b>Basic usage of a NetworkConnection is tested by many other testcases,
 * {@link PlayerTest} is a good example.
 */
public class NetworkConnectionTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(NetworkConnectionTest.class);
		
	public NetworkConnectionTest() {
		super();
	}
	public NetworkConnectionTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * As a first step, verify that we can create two NetworkConnections.
	 * <br>Each NetworkConnection is setup with a fake SDP sourced for the property "testcase.sipphoneSDP".
	 * Both NetworkConnections are then released.
	 */
	@AssertionIds( ids = {"javadoc::NetworkConnection::1" }, desc = "getSdpPortManager return a SdpPortManager")
	public void test_2_1_4_1_MultiNCs() throws Exception  {
		NetworkConnection NC1 = setupaNC();
		NetworkConnection NC2 = setupaNC();
		NC1.release();
		NC2.release();
	}

	/**
	 * Create a pair of NetworkConnections, exchanging their local/remote SDP.
	 * <br>
	 * <ol>
	 * <li>Create NC1 and NC2, using <code>NetworkConnectionConfig.BASIC</code> as configuration</li>
	 * <li>Work with SdpMgr1 and SdpMgr2, the SdpPortManagers of NC1 and NC2</li>
	 * <li>Ask for SdpMgr1 media server SDP, do not provide UserAgent SDP:  <code>SdpMgr1.generateSdpOffer()</code>
	 * <br> &nbsp;&nbsp;&nbsp;--> provide SDP1</li>
	 * <li>Ask for SdpMgr2 media server SDP, providing SdpMgr1.mediaServerSDP as UserAgentSDP: <code>SdpMgr2.processSdpOffer(SDP1)</code>
	 * <br> &nbsp;&nbsp;&nbsp;--> provide SDP2</li>
	 * <li>Provide SDP2 as UserAgent SDP for SdpMgr1: <code>SdpMgr1.processSdpAnswer(SDP2)</code></li>
	 * </ol>
	 * @throws Exception
	 */
	public void test_2_1_4_2_NCLoopback() throws Exception {
		setupLoopback(null, null, false);
	}

	/**
	 * Same setup as {@link #test_2_1_4_2_NCLoopback()}, then join a MediaGroup on each NetworkConnection, play on one and record on the other.
	 * <br>The recorded file is then compared with the originally played file.
	 * <br>Uses properties "testcase.promptfile" and "testcase.recordedfile".
	 * @throws Exception
	 */
	@AssertionIds( ids = { "javadoc::NetworkConnection::2", "javadoc::SdpPortManager::1", "javadoc::SdpPortManager::2", "javadoc::SdpPortManager::3"}, desc = "BASIC is able to handle network audio streams")
	public void test_2_1_4_3_PlayRecord() throws Exception {
		NetworkConnection[] myNCs = setupLoopback(null, null, false);
		MediaGroup myMG1 = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		MediaGroup myMG2 = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		myMG1.join(Direction.DUPLEX, myNCs[0]);
		myMG2.join(Direction.DUPLEX, myNCs[1]);
		
		final Player player = myMG1.getPlayer();
		assertNotNull(player);
		EventWaiter<PlayerEvent> playerW = new EventWaiter<PlayerEvent>(player, log);
		final Recorder recorder = myMG2.getRecorder();
		assertNotNull(recorder);
		recorder.addListener(new MediaEventListener<RecorderEvent>() {
			@Override
			public void onEvent(RecorderEvent anEvent) {
				log.info("MG2 recorder Received: "+anEvent);
			}			
		});
		
		URI played = URI.create(getProperty("testcase.promptfile"));
		URI recorded = URI.create(getProperty("testcase.recordedfile"));
		recorder.record(recorded, RTC.NO_RTC, Parameters.NO_PARAMETER);
		player.play(played, RTC.NO_RTC, Parameters.NO_PARAMETER);
		playerW.waitForOKEvent(20000);
		recorder.stop();
		assertTrue(filesAreIdentical(played, recorded));
	}
	
	private boolean filesAreIdentical(URI file1, URI file2) {
		return true; // TODO implement a real contents check
	}
}
