/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008-2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness;

import java.net.URI;

import javax.media.mscontrol.MediaEventListener;
import javax.media.mscontrol.MediaEventNotifier;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.networkconnection.SdpPortManager;
import javax.media.mscontrol.networkconnection.SdpPortManagerEvent;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.MediaSessionBase;
/**
 * Test that API listeners can be added/removed, and can execute any code.
 */
public class ListenerTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(ListenerTest.class);
	
	public ListenerTest() {
		super();
	}
	public ListenerTest(String testMethod) {
		super(testMethod);
	}
	
	public int finishedruns;

	
	private class MediaListener_for_2_2_3_1 implements MediaEventListener<PlayerEvent> {
		private boolean doRemoveListener;
		public MediaListener_for_2_2_3_1(boolean removelistener) {
			doRemoveListener=removelistener;
		}
		@Override
		public void onEvent(PlayerEvent anEvent) {
			MediaEventNotifier<PlayerEvent> notifyer = (MediaEventNotifier<PlayerEvent>)anEvent.getSource();
			log.debug("Listener of "+notifyer+" is called");
			if(doRemoveListener) {
				// self-removal of this listener
				try {
					notifyer.removeListener(this);
					log.debug("Listener is removed from "+notifyer);
				} catch (Exception e) {
					fail("Removing Listener in Listener's onEvent() callback method caused exception: "+e);
				}
			}
			synchronized (notifyer) {
				finishedruns++;
				log.debug("finishedruns="+finishedruns);
				notifyer.notify();
			}
		}
	}
	
	/**
	 * Test if a Listener can <code>removeListener</code> of itself. 
	 * <br>Two Listeners are attached to a Player.
	 * <br>After the 1st play, one of the two listeners will be removed. 
	 * <br>After the 2nd play, the listeners should be notified for the third time. 
	 */
	public void test_2_2_3_1_remove_Listener() throws Exception {
		
		// Player setup
		NetworkConnection myNC = setupaNC();
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		Player myPlayer = myMG.getPlayer();
		myNC.join(Direction.DUPLEX, myMG);

		// Add two listeners
		myPlayer.addListener(new MediaListener_for_2_2_3_1(true));
		myPlayer.addListener(new MediaListener_for_2_2_3_1(false));		

		// During the 1st run, listener 1 should be removed
		finishedruns=0;
		myPlayer.play(URI.create(getProperty("testcase.onesecondfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		synchronized (myPlayer) {
			// wait for one listener
			log.debug("Waiting for first listener");
			myPlayer.wait(5000);
			// wait for the other listener
			log.debug("Waiting for second listener");
			myPlayer.wait(5000);
		}
		assertEquals("Wrong number of listener calls: ", 2, finishedruns);
		// the 2nd run, only 1 Listener left.
		myPlayer.play(URI.create(getProperty("testcase.onesecondfile")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		synchronized (myPlayer) {
			// wait for 1st listener
			myPlayer.wait(5000);
			// will not be notified, expect time out after 5 seconds
			myPlayer.wait(5000);
		}
		assertEquals("Wrong number of listener calls: ", 3, finishedruns);
	}
	
	
	/**
	 * JSR309 implementation is in most case based on or deployed in application server
	 * such as a J2EE Server, SIP Server container. In most case, system resource as
	 * thread, memory are managed by the container (e.g. J2EE Spec). This test case 
	 * targets this issue. The case is blocking in listener for 40 seconds and checking
	 * any impact on rest of the application.   
	 * <br>  
	 * 10 NetworkConnections are created and processSdpOffer() methods are called to query the
	 * media server's SDP.  The thread which carries the event/response notifying listener
     * are blocked in the listener's onEvent() method. The notification counter and execution 
	 * time are verified to measure the impact on the overall performance.
    */

	public void test_2_2_3_2_Run_AnyCode_In_Listener() throws Exception {
		int ncnumber = 10;
		finishedruns=0;
		NetworkConnection[] ncs = new NetworkConnection[ncnumber];
		// prepare the NCs
		for(int i=0; i<ncnumber; i++) {
			ncs[i] = myMediaSession.createNetworkConnection(NetworkConnection.BASIC);
			ncs[i].getSdpPortManager().addListener(new MediaEventListener<SdpPortManagerEvent>() {
				@Override
				public void onEvent(SdpPortManagerEvent anEvent) {
					SdpPortManager notifyer = anEvent.getSource();
					// if(anEvent.getEventType().equals(SdpPortManager.MODIFY_COMPLETED)) {
					if(anEvent.getEventType().equals(SdpPortManagerEvent.ANSWER_GENERATED)) {
						finishedruns++;
					}
					log.debug(notifyer+" received: "+anEvent);
					try {
						Thread.sleep(40000);
					} catch (InterruptedException e) {
						
					}
				}			
			});
		}
		
		// Fire 10 modifies
		long actualDuration = System.currentTimeMillis();
		for(int i=0; i<ncnumber; i++) {
			ncs[i].getSdpPortManager().processSdpOffer(getProperty("testcase.sipphoneSDP").getBytes());
		}
		actualDuration = System.currentTimeMillis()-actualDuration;
		long expectedduration = 1000;
		log.debug("Duration to fire 10 modifies:"+actualDuration );
		assertTrue("Create 10 NetworkConnections takes "+actualDuration+" ms.", actualDuration<expectedduration);
	
		// Wait for the 200 response for NCs 
		Thread.sleep(10000);
		
		assertTrue(finishedruns+" out of 10 NetworkConnections are created.", finishedruns>=ncnumber);
	}
	
	
}
