/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness.MSFaultTestServer;

import javax.media.mscontrol.MediaErr;
import javax.media.mscontrol.MsControlException;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.networkconnection.SdpPortManager;
import javax.media.mscontrol.networkconnection.SdpPortManagerEvent;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.MediaSessionBase;

public class MediaServerFaultTest extends MediaSessionBase {

	static Logger log = Logger.getLogger(MediaServerFaultTest.class);

	public MediaServerFaultTest(){
		super();
	}
	public MediaServerFaultTest(String testMethod){
		super(testMethod);
	}

	protected SdpPortManagerEvent lastNCevent;

	/**    
	 * Media Server terminates the SIP session with BYE
	 * <ol>
	 * <li>Create a NetworkConnection. As the MS initiated BYE is received</li>
	 * <li>a MEDIA_SERVER_SDP_CHANGED event triggered the listener and</li>
	 * <li>the local SDP is cleared.</li>
	 * <li>Server emulator is com.hp.opencall.jmsc.test.mandatory.robustness.MSFaultTestServer.Test_ms_send_bye</li>
	 * </ol>
	 */
	public void test_2_2_5_1_MediaServerSendsBye() throws Exception {
		NetworkConnection myNC = myMediaSession.createNetworkConnection(NetworkConnection.BASIC);
		SdpPortManager myPorts = myNC.getSdpPortManager();
		EventWaiter<SdpPortManagerEvent> ncWaiter = new EventWaiter<SdpPortManagerEvent>(myPorts, log);
		myPorts.processSdpOffer(getProperty("testcase.sipphoneSDP").getBytes());
		ncWaiter.waitForOKEvent(3000); // wait for session established
		lastNCevent = ncWaiter.waitForAnyEvent(20000); // wait for BYE
		assertTrue("Receiving BYE fails.", lastNCevent.getEventType().equals(SdpPortManagerEvent.UNSOLICITED_OFFER_GENERATED)&&myPorts.getMediaServerSessionDescription()==null);
	}


	/**    
	 * Media Server rejects initial INIVITE carrying SDP, in most case modifying local SDP fails. 
	 * <ol>
	 * <li>Create a NetworkConnection. As the MS responses with 451 Media type not supported</li>
	 * <li>a Error.REFUSED event triggered the listener.</li>
	 * <li>Server emulator is com.hp.opencall.jmsc.test.mandatory.robustness.MSFaultTestServer.Test_MS_Reject_SDP</li>
	 * </ol>
	 */
	public void test_2_2_5_2_MediaServerRejectSDP() throws Exception {
		NetworkConnection myNC = myMediaSession.createNetworkConnection(NetworkConnection.BASIC);
		SdpPortManager myPorts = myNC.getSdpPortManager();
		EventWaiter<SdpPortManagerEvent> ncWaiter = new EventWaiter<SdpPortManagerEvent>(myPorts, log);
		//myPorts.modify(SdpPortManager.CHOOSE, getProperty("testcase.sipphoneSDP").getBytes());		
		myPorts.processSdpOffer(getProperty("testcase.sipphoneSDP").getBytes());		
		lastNCevent = ncWaiter.waitForAnyEvent(10000); // wait for 4XX response
		assertTrue("4XX response can not be processed.",lastNCevent.getError().equals(MediaErr.REFUSED));
		assertTrue("4XX response can not be processed.",lastNCevent.getEventType().equals(SdpPortManagerEvent.ANSWER_GENERATED));		
	}	


	/**    
	 * Media Server manipulates SIP Session by sending re-Invite. 
	 * <ol>
	 * <li>Create a NetworkConnection. As the MS initiated Re-Invite is received</li>
	 * <li>a MEDIA_SERVER_SDP_CHANGED event triggered the listener and</li>
	 * <li>a new local SDP is set. After the new remote SDP has been obtained, modify</li>
	 * <li>method is called again to inform MS the new remote SDP. </li>
	 * <li>This modify method is synchronized and returned as ACK is received.</li>
	 * <li>Server emulator is com.hp.opencall.jmsc.test.mandatory.robustness.MSFaultTestServer.test_ms_reInvite</li>
	 * </ol>
	 */
	public void test_2_2_5_3_MediaServerSendsReInvite() throws Exception {
		NetworkConnection myNC = myMediaSession.createNetworkConnection(NetworkConnection.BASIC);
		SdpPortManager myPorts = myNC.getSdpPortManager();
		EventWaiter<SdpPortManagerEvent> ncWaiter = new EventWaiter<SdpPortManagerEvent>(myPorts, log);
		myPorts.processSdpOffer(getProperty("testcase.sipphoneSDP").getBytes());
		ncWaiter.waitForOKEvent(3000); // wait for session established
		lastNCevent = ncWaiter.waitForAnyEvent(20000); // wait for re-Invite
		assertTrue("Receiving re-Invite fails.", lastNCevent.getEventType().equals(SdpPortManagerEvent.UNSOLICITED_OFFER_GENERATED)&&myPorts.getMediaServerSessionDescription()!=null);
		myPorts.processSdpAnswer(getProperty("testcase.sipphoneSDP").getBytes()); // Send 200 with "new" remote SDP - synchronized method, no event is waited.  
	}

	/**    
	 * Media Server rejects re-Invite for Mixer
	 * <ol>
	 * <li>Create a NetworkConnection and join with Mixer</li>
	 * <li>the re-Invite to configure the leg is rejected by MS</li>
	 * <li>a exception should warn the application.</li>
	 * </ol>
	 */

	public void test_2_2_5_4_MediaServerRejectReinvite() throws Exception {
		NetworkConnection myNC = setupaNC();
		final MediaMixer myMixer = myMediaSession.createMediaMixer(MediaMixer.AUDIO);
		try {
			myMixer.join(Direction.DUPLEX, myNC);
			fail("reinvite is rejected, but no exception.");
		} catch (MsControlException e) {
			log.debug("reinvite is rejected with exception "+e);
		}
	}

}
