/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness;

import javax.media.mscontrol.MediaObject;
import javax.media.mscontrol.MediaSession;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mixer.MediaMixer;
import javax.media.mscontrol.mixer.MixerAdapter;
import javax.media.mscontrol.networkconnection.NetworkConnection;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.AssertionIds;
import com.hp.opencall.jmsc.test.MediaSessionBase;

/**
 * Test invalid or odd values for {@link MediaObject#MEDIAOBJECT_ID}, for various
 * MediaObjects.
 */
public class MediaObjectIdTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(MediaObjectIdTest.class);
	
	public MediaObjectIdTest() {
		super();
	}
	public MediaObjectIdTest(String testMethod) {
		super(testMethod);
	}

	/**
	 * Test that a 30 characters {@link MediaObject#MEDIAOBJECT_ID} is accepted.
	 */
	@AssertionIds( ids = {"javadoc::MediaObject::2"})	
	public void test_2_2_10_1_LongId() throws Exception {
		MediaSession ms = myMsControlFactory.createMediaSession();
		assertNotNull(ms);
		log.info("MediaSession URI: \""+ms.getURI()+"\"");
		Parameters params = ms.createParameters();
		params.put(MediaObject.MEDIAOBJECT_ID, "MGThisIsAVeryLongName123456789");
		MediaGroup mg = ms.createMediaGroup(MediaGroup.PLAYER, params);
		assertNotNull(mg);
		log.info("Mediagroup URI: \""+mg.getURI()+"\"");
		params.put(MediaObject.MEDIAOBJECT_ID, "NCThisIsAVeryLongName123456789");
		NetworkConnection nc = ms.createNetworkConnection(NetworkConnection.BASIC, params);
		assertNotNull(nc);
		log.info("NetworkConnection URI: \""+nc.getURI()+"\"");
		params.put(MediaObject.MEDIAOBJECT_ID, "MXThisIsAVeryLongName123456789");
		MediaMixer mx = ms.createMediaMixer(MediaMixer.AUDIO, params);
		assertNotNull(mx);
		log.info("Mediagroup URI: \""+mx.getURI()+"\"");
		params.put(MediaObject.MEDIAOBJECT_ID, "MAThisIsAVeryLongName123456789");
		MixerAdapter ma = mx.createMixerAdapter(MixerAdapter.DTMF_CLAMP, params);
		assertNotNull(ma);		
		log.info("MixerAdapter URI: \""+ma.getURI()+"\"");
	}
	/**
	 * Test that a {@link MediaObject#MEDIAOBJECT_ID} started with a non-letter/non-number is rejected.
	 * <p>Tested characters: "*", " ", "-", ":", "_", "&gt;", "&lt;"
	 */
	@AssertionIds( ids = {"javadoc::MediaObject::1"})	
	public void test_2_2_10_2_InvalidInitialCharacter() throws Exception {
		Parameters params = createParameters();
		String[] invalidFirstChars = new String[] {"*", " ", "-", ":", "_", "<", ">"};
		for (String invalidFirstChar: invalidFirstChars) {
			params.put(MediaObject.MEDIAOBJECT_ID, invalidFirstChar+"invalidname");

			try {
				MediaGroup mg = myMediaSession.createMediaGroup(MediaGroup.PLAYER, params);
				assertNull("MediaGroup creation should have failed", mg);
			} catch (Exception e) {
				log.info("Caught expected exception: "+e.getMessage());
			}

			try {
				NetworkConnection nc = myMediaSession.createNetworkConnection(NetworkConnection.BASIC, params);
				assertNull("NetworkConnection creation should have failed", nc);
			} catch (Exception e) {
				log.info("Caught expected exception: "+e.getMessage());
			}

			try {
				MediaMixer mx = myMediaSession.createMediaMixer(MediaMixer.AUDIO, params);
				assertNull("MediaMixer creation should have failed", mx);
			} catch (Exception e) {
				log.info("Caught expected exception: "+e.getMessage());
			}

			MediaMixer mx = myMediaSession.createMediaMixer(MediaMixer.AUDIO, Parameters.NO_PARAMETER);

			try {
				MixerAdapter ma = mx.createMixerAdapter(MixerAdapter.DTMF_CLAMP, params);
				assertNull("MixerAdapter creation should have failed", ma);
			} catch (Exception e) {
				log.info("Caught expected exception: "+e.getMessage());
			}
		}
	}
	/**
	 * Test that duplicate {@link MediaObject#MEDIAOBJECT_ID}s are rejected.
	 */
	@AssertionIds( ids = {"javadoc::MediaObject::3"})
	public void test_2_2_10_3_DuplicateIDs() throws Exception {
		createObjects(false);
		createObjects(true);		
	}
	MediaMixer mixer = null;
	private void createObjects(boolean isIdAllocated) throws Exception {
		Parameters params = myMediaSession.createParameters();
		try {
			params.put(MediaObject.MEDIAOBJECT_ID, "MediaGroup-1");
			MediaGroup mg = myMediaSession.createMediaGroup(MediaGroup.PLAYER, params);
			if (isIdAllocated)
				assertNull("MediaGroup creation should have failed", mg);
			else
				assertNotNull(mg);
		} catch (Exception e) {
			if (isIdAllocated)
				log.info("Caught expected exception: "+e.getMessage());
			else
				throw e; // unexpected
		}
		try {
			params.put(MediaObject.MEDIAOBJECT_ID, "NetworkConnection-1");
			NetworkConnection nc = myMediaSession.createNetworkConnection(NetworkConnection.BASIC, params);
			if (isIdAllocated)
				assertNull("NetworkConnection creation should have failed", nc);
			else
				assertNotNull(nc);
		} catch (Exception e) {
			if (isIdAllocated)
				log.info("Caught expected exception: "+e.getMessage());
			else
				throw e; // unexpected
		}
		try {
			params.put(MediaObject.MEDIAOBJECT_ID, "MediaMixer-1");
			MediaMixer mx = myMediaSession.createMediaMixer(MediaMixer.AUDIO, params);
			if (isIdAllocated)
				assertNull("MediaMixer creation should have failed", mx);
			else {
				assertNotNull(mx);
				mixer = mx; // save for MA test later
			}
		} catch (Exception e) {
			if (isIdAllocated)
				log.info("Caught expected exception: "+e.getMessage());
			else
				throw e; // unexpected
		}
		try {
			params.put(MediaObject.MEDIAOBJECT_ID, "MixerAdapter-1");
			MixerAdapter ma = mixer.createMixerAdapter(MixerAdapter.DTMF_CLAMP, params);
			if (isIdAllocated)
				assertNull("MediaMixer creation should have failed", ma);
			else
				assertNotNull(ma);
		} catch (Exception e) {
			if (isIdAllocated)
				log.info("Caught expected exception: "+e.getMessage());
			else
				throw e; // unexpected
		}
	}
}
