/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness.mediagroup;

import java.net.URI;

import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.join.JoinException;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.mediagroup.Recorder;
import javax.media.mscontrol.mediagroup.RecorderEvent;
import javax.media.mscontrol.mediagroup.SpeechDetectorConstants;
import javax.media.mscontrol.mediagroup.signals.SignalDetector;
import javax.media.mscontrol.mediagroup.signals.SignalDetectorEvent;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.resource.RTC;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.MediaSessionBase;

/**
 * Testcases involving media operations on a Mediagroup that is not properly joined to a NetworkConnection or MediaMixer.
 *
 */
public class MediaGroupUnjoinTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(MediaGroupUnjoinTest.class);
	int recordCompletions;
    public MediaGroupUnjoinTest(){
    	super();
    }
    
    public MediaGroupUnjoinTest(String testMethod){
    	super(testMethod);
    }

     
	 /**    
	  * Play while MediaGroup is unjoined.
	  * <p>
	  * Two media groups are connected/joined with looped NetworkConnections. 
	  * One acts as player and the other MG as signal detector.<br><br>
	  *       MG(player)---->looped NCs----->MG(signal detector)<br><br>
	  * In the first run, signal is detected by the Signal detector after the player starts.
	  * Then the looped NC is unjoined from the player MG.
	  * In the second run, the player plays the same media file which contains the signal, 
	  * and no signal should be detected on the detector MG.
	  * Two behaviours are acceptable: either an exception, or playing as usual
	  * 
	  */
 	public void test_2_2_4_1_UnjoinMGPlay() throws Exception {
		
		NetworkConnection[] ncs = setupLoopback(null, null, true);
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> pwaiter = new EventWaiter<PlayerEvent>(myPlayer, log);
		final MediaGroup myDetector = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		myMG.join(Direction.DUPLEX, ncs[0]);
		myDetector.join(Direction.DUPLEX, ncs[1]);
		final SignalDetector mySD = myDetector.getSignalDetector();
		EventWaiter<SignalDetectorEvent> sdW = new EventWaiter<SignalDetectorEvent>(mySD, log);
		
		// Signal should be detected when joined.
		log.info("Detecting DTMF with a joined MediaGroup");
		mySD.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		myPlayer.play(new URI(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sdE = sdW.waitForOKEvent(5000);
		assertEquals("Bad SignalDetectorEvent qualifier:", SignalDetectorEvent.NUM_SIGNALS_DETECTED, sdE.getQualifier());
		pwaiter.waitForOKEvent(5000);
		
		// after the looped NCs is unjoined from the player's MG, 
		// signal should not be detected anymore
		log.info("Checking that DTMF is not detected when MediaGroup is unjoined");
		myMG.unjoin(ncs[0]);
		mySD.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		// Two behaviours are acceptable: either an exception, or playing as usual
		try {
			myPlayer.play(new URI(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
			sdW.failIfEvent(3000); // must be shorter than P_InitialTimeout
		} catch (JoinException je) {
			log.info("Caught acceptable JoinException: "+je.getMessage());
		}
	}

     
 	 /**    
 	  * Record while MediaGroup is unjoined.
 	  * <p>
 	  * Two media groups are connected/joined with looped NetworkConnections. 
 	  * One acts as player and the other MG as recorder.<br><br>
 	  *       MG(player)---->looped NCs----->MG(recorder)<br><br>
 	  * In the first run, the recorder receives stream from player. 
 	  * Then the looped NC is unjoined from the recorder MG.
 	  * By the second run, the recorder can not receive stream from player via NCs anymore,
 	  * The recorder can record silence or throw a JoinException, depending on the implementation.
 	  */

	public void test_2_2_4_2_UnjoinMGRecord() throws Exception {
		// Prepare player and reorder

		NetworkConnection[] ncs = setupLoopback(null, null, false);
		final MediaGroup myMG1 = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		final MediaGroup myMG2 = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		myMG1.join(Direction.DUPLEX, ncs[0]);
		myMG2.join(Direction.DUPLEX, ncs[1]);
		
		final Recorder myRecorder = myMG2.getRecorder();
		EventWaiter<RecorderEvent> rwaiter = new EventWaiter<RecorderEvent>(myRecorder, log);
		final Player myPlayer = myMG1.getPlayer();
		EventWaiter<PlayerEvent> pwaiter = new EventWaiter<PlayerEvent>(myPlayer, log);
		URI promptfile = URI.create(getProperty("testcase.shortpromptfile"));
		URI file = URI.create(getProperty("testcase.recordedfile"));
		
		// First run, recorder is connected with player.
		int finalTimeout = 3000;
		int expectedRecordDuration = 3000+finalTimeout; // shortpromptfile has a duration of 3 seconds
		Parameters recordOpt = createParameters();
		recordOpt.put(Recorder.SILENCE_TERMINATION_ON, true);
		recordOpt.put(SpeechDetectorConstants.FINAL_TIMEOUT, finalTimeout);
		myRecorder.record(file, RTC.NO_RTC, recordOpt);
		myPlayer.play(promptfile, RTC.NO_RTC, Parameters.NO_PARAMETER);
		rwaiter.failIfEvent(expectedRecordDuration-1000); // with a 1s margin
		RecorderEvent recEvt = rwaiter.waitForOKEvent(5000); // wait for final timeout
		pwaiter.waitForOKEvent(4000); 
		
		// Second run, recorder is unjoined from player
		// Either the record method throws a JoinException,
		// or the record completes after the finalTimeout, with the qualifier SILENCE
		myMG2.unjoin(ncs[1]);
		try {
			myRecorder.record(file, RTC.NO_RTC, recordOpt);
			myPlayer.play(promptfile, RTC.NO_RTC, Parameters.NO_PARAMETER);
			recEvt = rwaiter.waitForOKEvent(finalTimeout+2000); // wait for final timeout 
			assertEquals("Bad RecorderEvent qualifier", RecorderEvent.SILENCE, recEvt.getQualifier());
		} catch (JoinException je) {
			// this is acceptable, no further processing needed
		}
	}
	
	 /**    
	  * Detecting signals while the MediaGroup is unjoined.
	  * <p>
	  * Two MediaGroups are connected/joined with looped NetworkConnections. 
	  * <br>One acts as player and the other as signal detector.
	  * <br>      MG(player)---->looped NCs----->MG(signal detector)
	  * <br>In the first run, a signal is detected by the Signal detector after the player starts.
	  * <br>Then the looped NC is unjoined from the Detector MG.
	  * <br>On the second run, the player plays the same media file which contains the signal, 
	  * and no signal should be detected on the detector MG:
	  * <br>Either receiveSignals throws a JoinException, or it completes normally with INITIAL_TIMEOUT_EXCEEDED.
	  * 
	  */
	public void test_2_2_4_3_UnjoinMGSignalDetector() throws Exception {
		
		NetworkConnection[] ncs = setupLoopback(null, null, true);
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		Player myPlayer = myMG.getPlayer();
		EventWaiter<PlayerEvent> pwaiter = new EventWaiter<PlayerEvent>(myPlayer, log);
		final MediaGroup myDetector = myMediaSession.createMediaGroup(MediaGroup.PLAYER_RECORDER_SIGNALDETECTOR);
		myMG.join(Direction.DUPLEX, ncs[0]);
		myDetector.join(Direction.DUPLEX, ncs[1]);
		final SignalDetector mySD = myDetector.getSignalDetector();
		EventWaiter<SignalDetectorEvent> sdW = new EventWaiter<SignalDetectorEvent>(mySD, log);
		
		// Signal should be detected as joined.
		mySD.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
		myPlayer.play(new URI(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
		SignalDetectorEvent sdE = sdW.waitForOKEvent(5000);
		assertEquals("Bad SignalDetectorEvent qualifier:", SignalDetectorEvent.NUM_SIGNALS_DETECTED, sdE.getQualifier());
		pwaiter.waitForOKEvent(5000);
		
		// After the SignalDetector is unjoined from the NetworkConnection, 
		// the signal should not be detected anymore.
		// Either receiveSignals throws a JoinException, or it completes normally with INITIAL_TIMEOUT_EXCEEDED.
		myDetector.unjoin(ncs[1]);
		try {
			mySD.receiveSignals(1, SignalDetector.NO_PATTERN, RTC.NO_RTC, Parameters.NO_PARAMETER);
			myPlayer.play(new URI(getProperty("testcase.dtmf1")), RTC.NO_RTC, Parameters.NO_PARAMETER);
			sdE = sdW.waitForOKEvent(10000);
			assertEquals("Bad SignalDetectorEvent qualifier:", SignalDetectorEvent.INITIAL_TIMEOUT_EXCEEDED, sdE.getQualifier());
		} catch (JoinException je) {
			// this is acceptable, no further processing needed
		}
	}	
}

