/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2008-2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness.mediagroup;

import java.net.URI;

import javax.media.mscontrol.MediaErr;
import javax.media.mscontrol.Parameters;
import javax.media.mscontrol.Qualifier;
import javax.media.mscontrol.join.Joinable.Direction;
import javax.media.mscontrol.mediagroup.MediaGroup;
import javax.media.mscontrol.mediagroup.Player;
import javax.media.mscontrol.mediagroup.PlayerEvent;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.resource.RTC;
import javax.media.mscontrol.resource.ResourceEvent;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.MediaSessionBase;
/**
 * Testcases with invalid arguments or Parameters to {@link Player#play(URI, RTC[], Parameters)}.
 * <br>Test also with invalid Player state.
 */
public class PlayerRobustnessTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(PlayerRobustnessTest.class);
	
	public PlayerRobustnessTest() {
		super();
	}
	public PlayerRobustnessTest(String testMethod) {
		super(testMethod);
	}
	
	/**
	 * Allowance in duration measurements
	 */
	int tolerance = 1500; // allow 1.5 second of error, depends on testing environment 

	class PlayerControl {
		Player player;
		EventWaiter<PlayerEvent> waiter;
	}
	
	private PlayerControl setupPlayer() throws Exception {
		PlayerControl result = new PlayerControl();
		NetworkConnection myNC = setupaNC();
		final MediaGroup myMG = myMediaSession.createMediaGroup(MediaGroup.PLAYER);
		myNC.join(Direction.DUPLEX, myMG);
		result.player = myMG.getPlayer();
		result.waiter = new EventWaiter<PlayerEvent>(result.player, log);
		return result;
	}

	/**
	 * Test that a bad URI (does not exist) is properly handled by the Player.
	 * <ol>
	 * <li>The URI does not exist</li>
	 * <br>Verify that error is {@link MediaErr#NOT_FOUND}.
	 * <br>verify that qualifier is null
	 * <li>The schema of the URI is not supported</li>
	 * <br>Verify that error is {@link MediaErr#BAD_ARG}.
	 * <br>verify that qualifier is null
	 * </ol>
	 * <p>Requires testcase.promptfile, less that 20 seconds duration.
	 */
	public void test_2_2_1_1_Play_BadURI() throws Exception {
		
		PlayerControl pctl = setupPlayer();
		
		// Test with non existing file
		pctl.player.play(URI.create("file:///opt/OC/data/jsr309-TCK-media/noSuchFileAtAll.wav"), RTC.NO_RTC, Parameters.NO_PARAMETER);
		PlayerEvent playEvt = pctl.waiter.waitForAnyEvent(5000);
		assertEquals("Wrong error code", MediaErr.NOT_FOUND, playEvt.getError());
		assertEquals("Bad Qualifier: ", ResourceEvent.NO_QUALIFIER, playEvt.getQualifier());
		
		// Test with invalid scheme
		pctl.player.play(URI.create("nosuchscheme:///opt/OC/data/jsr309-TCK-media/noSuchFileAtAll.wav"), RTC.NO_RTC, Parameters.NO_PARAMETER);
		playEvt = pctl.waiter.waitForAnyEvent(5000);
		assertEquals("Wrong error code", MediaErr.BAD_ARG, playEvt.getError());
		assertEquals("Bad Qualifier: ", ResourceEvent.NO_QUALIFIER, playEvt.getQualifier());
	}
	/**
	* Check behavior when an invalid value for {@link Player#MAX_DURATION} is provided.
	* <p>The implementation should revert to the default value (infinite) in this case.
	* <ol>
	* <li>Play a file (that must be at least 5 seconds long) with MAX_DURATION set to -10 seconds</li>
	* <li>Verify that the qualifier of the PlayerEvent is {@link PlayerEvent#END_OF_PLAY_LIST},
	*      and the played duration is the full duration of the file.</li>
	* <li>In the second run the Maxduration is set to double the file actual duration.</li>
	* </ol>
	*/
	public void test_2_2_1_2_Player_Invalid_Maxduration() throws Exception {	
		
		PlayerControl pctl = setupPlayer();
		
		URI file = URI.create(getProperty("testcase.promptfile"));

        int promptDuration = (int)measurePlayDuration(file, null, log);
        
		// 1st run, with maxduration -10 seconds
		Integer maxduration = -10000;  
		Parameters options = createParameters();
		options.put(Player.MAX_DURATION, maxduration);
		long duration = System.currentTimeMillis(); // initialize
		pctl.player.play(file, RTC.NO_RTC, options);
		PlayerEvent playerEvt = pctl.waiter.waitForOKEvent(promptDuration*2);
		duration = System.currentTimeMillis()-duration; // measure
		checkDuration("Measured play", promptDuration, duration, tolerance, log);
		assertEquals("Bad play completion qualifier: ", PlayerEvent.END_OF_PLAY_LIST, playerEvt.getQualifier());
		
		// 2nd run, with maxduration double of the file actual duration
		options.put(Player.MAX_DURATION, promptDuration*2);
		duration = System.currentTimeMillis(); // initialize
		pctl.player.play(file, RTC.NO_RTC, options);
		playerEvt = pctl.waiter.waitForOKEvent(promptDuration*2);
		duration = System.currentTimeMillis()-duration; // measure
		checkDuration("Measured play", promptDuration, duration, tolerance, log);
		assertEquals("Bad play completion qualifier: ", PlayerEvent.END_OF_PLAY_LIST, playerEvt.getQualifier());
	}

	
	/**
	* Check behavior as {@link Player#START_OFFSET} is set longer than the file maximum play duration.
	* <br>The maximum duration of the file will be set as value of START_OFFSET
	* <ol>
	* <li>Play a file with START_OFFSET = 50000, which is clearly greater than maximum duration of the file.</li>
	* <li>Verify that the duration is very short (ideally zero).</li>
	* <li>Verify that the qualifier of the PlayerEvent is {@link PlayerEvent#END_OF_PLAY_LIST}.</li>
	* </ol>
	*/
	public void test_2_2_1_3_Player_Invalid_StartOffset() throws Exception {
		
		PlayerControl pctl = setupPlayer();
		
		URI file = URI.create(getProperty("testcase.promptfile"));
	
		int offsetValue = 50000;  // 50 seconds
		Parameters options = createParameters();
		options.put(Player.START_OFFSET, offsetValue);

		long duration = System.currentTimeMillis();
		pctl.player.play(file, RTC.NO_RTC, options);
		PlayerEvent playerEvt = pctl.waiter.waitForOKEvent(20000);
		duration = System.currentTimeMillis()-duration;
		// duration should be very short (zero ideally).
		// We compare against 1 second
		checkDuration("Measured play", 1000, duration, 2000, log);
		assertEquals("Bad play completion qualifier: ", PlayerEvent.END_OF_PLAY_LIST, playerEvt.getQualifier());
	}
	
	
	/**
	* Check behavior as {@link Player#REPEAT_COUNT} is set with invalid type (String) and value.
	* <ol>
	* <li>Play a one-second file with REPEAT_COUNT = -10 or REPEAT_COUNT="one"</li>
	* <li>Verify the actual Duration is 1 second, the file is played 1 time which is the default value.</li>
	* <li>verify that the qualifier of the PlayerEvent is {@link PlayerEvent#END_OF_PLAY_LIST}.</li>
	* </ol>
	*/
	public void test_2_2_1_4_Invalid_RepeatCount() throws Exception {
		
		// Test with String as value
		String repeatCount = "one";  
		URI oneSecFile = URI.create(getProperty("testcase.onesecondfile"));
		Parameters options = createParameters();
		options.put(Player.REPEAT_COUNT, repeatCount);
		long actualDuration = measurePlayDuration(oneSecFile, options, log);
		checkDuration("Measured play", 1000, actualDuration, tolerance, log);
		Qualifier qualifier = lastPlayerEvent.getQualifier();
		assertEquals("Bad PlayerEvent qualifier: ", PlayerEvent.END_OF_PLAY_LIST, qualifier);
		
		// Test with -10
		options.put(Player.REPEAT_COUNT, -10);
		actualDuration = measurePlayDuration(oneSecFile, options, log);
		checkDuration("Measured play", 1000, actualDuration, tolerance, log);
		qualifier = lastPlayerEvent.getQualifier();
		assertEquals("Bad PlayerEvent qualifier: ", PlayerEvent.END_OF_PLAY_LIST, qualifier);		
	}

	/**
	* Check behavior when {@link Player#INTERVAL} is set with an invalid value.
	* <br>Play a one-second file with REPEAT_COUNT = 2, and INTERVAL = -3s.
	* <br>Verify that the play duration is 2 seconds, since the default value for INTERVAL is zero.
	*/
	public void test_2_2_1_5_Invalid_Interval() throws Exception {
		int interval = -3000; // invalid value 
		int repeatCount = 2;
		URI oneSecFile = URI.create(getProperty("testcase.onesecondfile"));
		Parameters options = createParameters();
		options.put(Player.REPEAT_COUNT, repeatCount);
		options.put(Player.INTERVAL, interval);
		long actualDuration = measurePlayDuration(oneSecFile, options, log);
		long expectedDuration = 1000*repeatCount;
		checkDuration("Measured play", expectedDuration, actualDuration, tolerance, log);
	}
	

	/**
	 * Verify that the Player sends a PlayerEvent only when appropriate.
	 * <p>The Player starts a play action and is stopped while the file is playing
	 * by invoking stop() method. The state of the Player should change to idle.
	 * After that a second play action is started, the file
	 * is played to completion and player received an EOD response. The stop() method
	 * is called again as the player is in idle state, no response/event should
	 * occur this time. 
	 */
	public void test_2_2_1_6_Player_State() throws Exception {
		
		PlayerControl pctl = setupPlayer();
		
		URI file = URI.create(getProperty("testcase.promptfile"));


		// Stop the play as it is not finished, player state transfers to idle 
		pctl.player.play(file, RTC.NO_RTC, Parameters.NO_PARAMETER);
		pctl.waiter.failIfEvent(3000);
		pctl.player.stop(true);
		PlayerEvent evt = pctl.waiter.waitForOKEvent(5000);
		assertEquals("Wrong PlayerEvent qualifier ", PlayerEvent.STOPPED, evt.getQualifier());
		
		// Wait until the player reaches the end of file and sends a EOD event.
		pctl.player.play(file, RTC.NO_RTC, Parameters.NO_PARAMETER);
		evt = pctl.waiter.waitForOKEvent(20000);
		assertEquals("Wrong PlayerEvent qualifier ", PlayerEvent.END_OF_PLAY_LIST, evt.getQualifier());
		
		// Stop again, no event expected
		pctl.player.stop(true);
		pctl.waiter.failIfEvent(5000);
	}
	
}
