/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Copyright (c) 2009 Hewlett-Packard, Inc. All rights reserved.
 * Use is subject to license terms.
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package com.hp.opencall.jmsc.test.mandatory.robustness.networkconnection;

import javax.media.mscontrol.networkconnection.CodecPolicy;
import javax.media.mscontrol.networkconnection.NetworkConnection;
import javax.media.mscontrol.networkconnection.SdpPortManager;
import javax.media.mscontrol.networkconnection.SdpPortManagerEvent;
import javax.media.mscontrol.networkconnection.SdpPortManagerException;

import org.apache.log4j.Logger;

import com.hp.opencall.jmsc.test.MediaSessionBase;
// modify in-use policy
// lowercase codec names
/**
 * This Test verifies the behaviour of the implementation versus application errors:
 * <ul>
 * <li>Invalid {@link CodecPolicy}</li>
 * <li>Bogus codec names</li>
 * </ul>
 */
public class CodecPolicyTest extends MediaSessionBase {
	static Logger log = Logger.getLogger(CodecPolicyTest.class);

	public CodecPolicyTest() {
		super();
	}
	public CodecPolicyTest(String method) {
		super(method);
	}
	private void setInvalidPolicy(CodecPolicy policy) throws Exception {
		// Show what we are applying
		showCodecPolicy(policy, log);
		// Create and setup the SdpPortManager
		SdpPortManager myPorts = myMediaSession.createNetworkConnection(NetworkConnection.BASIC).getSdpPortManager();
		try {
			myPorts.setCodecPolicy(policy);		
			fail("PCMU is both required and excluded, should cause SdpPortManagerException");
		} catch (SdpPortManagerException e) {
			log.info("Caught expected SdpPortManagerException: "+e.getMessage());
		}		
	}
	/**
	 * Test that a codec cannot be at the same time required and excluded.
	 * <br>"PCMU" is chosen for this testcase.
	 */
	public void test_2_2_8_1_ExcludedInRequired() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setRequiredCodecs(new String[]{"telephone-event","PCMU"});
		myPolicy.setExcludedCodecs(new String[] {"PCMU", "G722"});
		setInvalidPolicy(myPolicy);
	}
	/**
	 * Test that a codec cannot be at the same time in the capabilities and excluded.
	 * <br>"PCMU" is chosen for this testcase.
	 */
	public void test_2_2_8_2_ExcludedInCapabilities() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setCodecCapabilities(new String[]{"telephone-event","PCMU"});
		myPolicy.setExcludedCodecs(new String[] {"PCMU", "G722"});
		setInvalidPolicy(myPolicy);
	}
	/**
	 * Test that a codec cannot be at the same time preferred and excluded.
	 * <br>"PCMU" is chosen for this testcase.
	 */
	public void test_2_2_8_3_ExcludedInPreferences() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setCodecPreferences(new String[]{"telephone-event","PCMU"});
		myPolicy.setExcludedCodecs(new String[] {"PCMU", "G722"});
		setInvalidPolicy(myPolicy);
	}
	private void setBogusPolicy(CodecPolicy policy) throws Exception {
		// Show what we are applying
		showCodecPolicy(policy, log);
		String richOffer = com.hp.opencall.jmsc.test.mandatory.functional.networkconnection.CodecPolicyTest.richOffer;
		// Create and setup the SdpPortManager
		SdpPortManager myPorts = myMediaSession.createNetworkConnection(NetworkConnection.BASIC).getSdpPortManager();
		myPorts.setCodecPolicy(policy);
		EventWaiter<SdpPortManagerEvent> waiter = new EventWaiter<SdpPortManagerEvent>(myPorts, log);
		myPorts.processSdpOffer(richOffer.getBytes());
		// Wait for the setup to complete
		SdpPortManagerEvent event = waiter.waitForOKEvent(3000);
		// The media server returned an SDP, let us check it
		checkSdpAnswer(new String(event.getMediaServerSdp()), policy, log);
	}
	/**
	 * Test that excluded codec names like "fmtp", "AS", "sendrecv", "audio", "IP4" are properly
	 * ignored by the implementation.
	 */
	public void test_2_2_8_4_BogusCodecNamesExcluded() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setExcludedCodecs(new String[] {"fmtp", "AS", "sendrecv", "audio", "IP4", "G722"});
		setBogusPolicy(myPolicy);
	}
	/**
	 * Test that codec capability names like "fmtp", "AS", "sendrecv", "audio", "IP4" are properly
	 * ignored by the implementation.
	 */
	public void test_2_2_8_5_BogusCodecNamesCapable() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setCodecCapabilities(new String[]{"fmtp", "AS", "sendrecv", "audio", "IP4", "telephone-event","PCMU"});
		setBogusPolicy(myPolicy);
	}
	/**
	 * Test that preferred codec names like "fmtp", "AS", "sendrecv", "audio", "IP4" are properly
	 * ignored by the implementation.
	 */
	public void test_2_2_8_6_BogusCodecNamesPreferred() throws Exception {
		CodecPolicy myPolicy = new CodecPolicy();
		myPolicy.setCodecPreferences(new String[]{"fmtp", "AS", "sendrecv", "audio", "IP4", "telephone-event","PCMU"});
		setBogusPolicy(myPolicy);
	}
}
